/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package com.adventnet.jmx.utils;

/**
 * This is a utility class which sorts a objectID array using
 * quick sort mechanism .By default the compareTo method compares
 * two strings str1 & str2 considering it to be a OID string. API
 * users may override this method inorder to have their own compareTo
 * method incase of different type of string.
 * <p>
 * Usage is :
 * <li>sorter = new Sorter();
 * <li>toSort = new String[2]{".1.3.6.1.3.2", ".1.3.6.1.3.12",};
 * <li>sorter.sort(toSort, null);
 */
public class Sorter implements java.io.Serializable
{


   /** This is a generic version of C.A.R Hoare's Quick Sort
    * algorithm.  This will handle arrays that are already
    * sorted, and arrays with duplicate keys.<BR>
    *
    * If you think of a one dimensional array as going from
    * the lowest index on the left to the highest index on the right
    * then the parameters to this function are lowest index or
    * left and highest index or right.  The first time you call
    * this function it will be with the parameters 0, a.length - 1.
    *
    * @param a       an integer array
    * @param lo0     left boundary of array partition
    * @param hi0     right boundary of array partition
    */
   public void QuickSort(String a[], int lo0, int hi0) throws Exception
   {
      int lo = lo0;
      int hi = hi0;
      String mid;

      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
	     while( ( lo < hi0 ) && (compareTo(a[lo], mid) < 0))
		 ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
	     while( ( hi > lo0 ) && (compareTo(a[hi], mid) > 0))
		 --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               swap(a, lo, hi);
               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            QuickSort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            QuickSort( a, lo, hi0 );

      }
   }

   /**
    * This method compares two string considering it be
	* OID string. API users may override this method incase
	* if they want to have different compareTo method.
	*/

    private int compareTo(String str1, String str2)
	{
		int ret = str1.compareTo(str2);

		if(ret > 0)
			return 1;
		if(ret < 0)
			return -1;
		return 0;
	}

   private void swap(String a[], int i, int j)
   {
      Object T;
      T = a[i];
      a[i] = a[j];
      a[j] = (String)T;

	  if(obj != null)
	  {

		int max = obj.length;

		for( int pos = 0; pos < max; pos++)
		{
			Object[] b = obj[pos];
			T = b[i];
			b[i] = b[j];
			b[j] = T;
		}
	  }


   }

   Object[][] obj = null;

   /**
    * This method sorts the OID string array in ascending order.
	* Also it sorts multi array objects in "obj" according to the key
	* OID string array "a".
	* @param a the key string array to be sorted.
	* @param obj the multiarray objects which are sorted according to the
	* key oid.
	*/
   public void sort(String a[], Object[][] obj) throws Exception
   {
	  this.obj = obj;
      QuickSort(a, 0, a.length - 1);
   }


   /** This is a generic version of C.A.R Hoare's Quick Sort
    * algorithm.  This will handle arrays that are already
    * sorted, and arrays with duplicate keys.<BR>
    *
    * If you think of a one dimensional array as going from
    * the lowest index on the left to the highest index on the right
    * then the parameters to this function are lowest index or
    * left and highest index or right.  The first time you call
    * this function it will be with the parameters 0, a.length - 1.
    *
    * @param a       an integer array
    * @param lo0     left boundary of array partition
    * @param hi0     right boundary of array partition
    */
   public void QuickSort(int a[], int lo0, int hi0) throws Exception
   {
	   int lo = lo0;
	   int hi = hi0;
	   int mid;

	   if ( hi0 > lo0)
	   {

	   /* Arbitrarily establishing partition element as the midpoint of
	   * the array.
		   */
		   mid = a[ ( lo0 + hi0 ) / 2 ];

		   // loop through the array until indices cross
		   while( lo <= hi )
		   {
		   /* find the first element that is greater than or equal to
		   * the partition element starting from the left Index.
			   */
			   while( ( lo < hi0 ) &&  ( a[lo] < mid ))
				   ++lo;

				   /* find an element that is smaller than or equal to
				   * the partition element starting from the right Index.
			   */
			   while( ( hi > lo0 ) &&  ( a[hi] > mid ))
				   --hi;

			   // if the indexes have not crossed, swap
			   if( lo <= hi )
			   {
				   swap(a, lo, hi);
				   ++lo;
				   --hi;
			   }
		   }

		   /* If the right index has not reached the left side of array
		   * must now sort the left partition.
		   */
		   if( lo0 < hi )
			   QuickSort( a, lo0, hi );

			   /* If the left index has not reached the right side of array
			   * must now sort the right partition.
		   */
		   if( lo < hi0 )
			   QuickSort( a, lo, hi0 );

	   }
   }

   private void swap(int a[], int i, int j)
   {
	   int T;
	   T = a[i];
	   a[i] = a[j];
	   a[j] = T;

   }

   public void sort(int a[]) throws Exception
   {
	   QuickSort(a, 0, a.length - 1);
   }
}