/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

/**
 * This interface is used to gain access to descriptors of the Descriptor class
 * which are associated with a JMX component,
 * i.e. MBean, MBeanInfo,  MBeanAttributeInfo, MBeanNotificationInfo,
 * MBeanOperationInfo, MBeanParameterInfo.
 * <p>
 * ModelMBeans make extensive use of this interface in ModelMBeanInfo classes.
**/
public interface DescriptorAccess
{
	/**
	 * Returns a copy of Descriptor.
	 *
	 * @return Descriptor associated with the component implementing this
	 * 				interface. Null should never be returned. At a minimum a
	 * 				default descriptor with the descriptor name and
	 * 				descriptorType should be returned.
	 */
	public Descriptor getDescriptor();

	/**
	 * Sets Descriptor (full replace).
	 *
	 * @param inDescriptor replaces the Descriptor associated with the
	 * 				component implementing this interface. If the inDescriptor
	 * 				is invalid for the type of Info object it is being set for,
	 * 				an exception is thrown. If the inDescriptor is null, then
	 * 				the Descriptor will revert to its default value which should
	 * 				contain, at a minimum, the descriptor name and descriptorType.
	 */
	public void setDescriptor(Descriptor inDescriptor);
}