/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * Defines the methods that should be implemented by a Dynamic MBean
 * (MBean that exposes a dynamic management interface).
 */
public abstract interface DynamicMBean
{
    /**
     * Provides the exposed attributes and actions of the Dynamic MBean using
	 * an MBeanInfo object.
     *
     * @return  An instance of <CODE>MBeanInfo</CODE> allowing all attributes
	 * 				and actions exposed by this Dynamic MBean to be retrieved.
     */
	public MBeanInfo getMBeanInfo();

	/**
     * Obtains the value of a specific attribute of the Dynamic MBean.
     *
	 * @param attribute - The name of the attribute to be retrieved
	 *
	 * @return The value of the attribute retrieved.
	 *
     * @exception AttributeNotFoundException - The specified attribute
	 * 				does not exist or cannot be retrieved.
	 *
     * @exception MBeanException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown by the MBean's getter.
     *
     * @exception ReflectionException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown while trying to invoke the getter.
	 */
	public Object getAttribute(String attribute)
						throws  AttributeNotFoundException,
								MBeanException,
								ReflectionException;

	/**
	 * Sets the value of a specific attribute of the Dynamic MBean.
	 *
	 * @param attribute - The identification of the attribute to be set and
	 * 				the value it is to be set to.
	 *
	 * @exception AttributeNotFoundException -  The specified attribute
	 * 				does not exist or cannot be retrieved.
	 *
	 * @exception InvalidAttributeValueException -  The specified value is
	 * 				not a valid value for the attribute.
	 *
	 * @exception MBeanException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown by the MBean's setter.
	 *
	 * @exception ReflectionException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown while trying to invoke the MBean's setter.
	 */
	public void setAttribute(Attribute attribute)
						throws  AttributeNotFoundException,
								InvalidAttributeValueException,
								MBeanException,
								ReflectionException;

    /**
     * Enables the values of several attributes of the Dynamic MBean.
     *
     * @param attributes A list of the attributes to be retrieved.
     *
     * @return  The list of attributes retrieved.
     *
     */
	public AttributeList getAttributes(String[] attributes);

    /**
     * Sets the values of several attributes of the Dynamic MBean.
     *
     * @param name The object name of the MBean within which the attributes
	 * 				are to be set.
	 *
     * @param attributes A list of attributes: The identification of the
	 * 				attributes to be set and  the values they are to be set to.
     *
     * @return  The list of attributes that were set, with their new values.
     */
	public AttributeList setAttributes(AttributeList attributes);

	/**
	 * Allows an action to be invoked on the Dynamic MBean.
	 *
	 * @param actionName - The name of the action to be invoked.
	 *
	 * @param params - An array containing the parameters to be set when
	 * 				the action is invoked.
	 *
	 * @param signature - An array containing the signature of the action.
	 * 				The class objects will be loaded through the same class
	 * 				loader as the one used for loading the MBean on which
	 * 				the action is invoked.
	 *
	 * @return The object returned by the action, which represents the result
	 * 				of invoking the action on the MBean specified.
	 *
	 * @exception MBeanException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown by the MBean's invoked method.
	 *
	 * @exception ReflectionException - Wraps a <CODE>java.lang.Exception</CODE>
	 * 				thrown while trying to invoke the method
	 */
	public Object invoke(String actionName, Object[] params, String[] signature)
						throws  MBeanException,
								ReflectionException;
}