/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;

/**
 * Provides general information for an MBean descriptor object.
 * The feature described can be:
 * <ul>
 * <li> an attribute,
 * <li> an operation,
 * <li> a parameter,
 * <li> a returned value or
 * <li> a notification.
 * </ul>
 */
public class MBeanFeatureInfo implements java.io.Serializable
{
	/* Serial version UID */
	private static final long serialVersionUID = 0x36db75dcc2c15521L;

	/**
	 * The name of the feature.
     */
	protected String name = null;

	/**
	 *  The description of the feature in a String format
	 */
	protected String description = null;

	/**
	 * Constructs a MBeanFeatureInfo object.
	 *
     * @param name The name of the feature.
     *
     * @param description A human readable description of the feature.
	 */
	public MBeanFeatureInfo(String name, String description)
	{
		this.name = name;
		this.description = description;
	}

	/**
	 * Returns the human readable description of the feature.
	 *
	 * @return the human readable description of the feature as String.
	 */
	public String getDescription()
	{
		return description;
	}

	/**
 	 * Returns the name of the feature.
 	 *
	 * @return the name of the feature.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Returns a human readable version of the MBeanFeatureInfo instance
	 * @return MBeanFeatrueInfo is returned in a human readable form (i.e) as a String
	 */
	public String toString()
	{
		return (" Name = "+ name +
				"\n Description = "+ description);
	}
}