/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;

/**
 * Describes describes a notification emitted by an MBean.
 */
public class MBeanNotificationInfo extends MBeanFeatureInfo
								implements Serializable, Cloneable
{
    /**
     * The type of the notification.
     */
	private String[] notifTypes = null;

	/* Serial version UID */
	private static final long serialVersionUID = 0xca09baa9dd73fd48L;

	private static final ObjectStreamField[] serialPersistentFields =
				{
					new ObjectStreamField("types",java.lang.String.class)
				};

	/**
     * Constructs an <CODE>MBeanNotificationInfo</CODE> object.
     *
     * @param notifType The event type string (in dot notation).
     *
     * @param name The name of the <CODE>Notification</CODE> class.
     *
     * @param description A human readable description of the data.
	 */
	public MBeanNotificationInfo(String[] notifTypes,
							     String name,
								 String description)
	{
		super(name, description);
		this.notifTypes = notifTypes;
	}

	/**
	 * Creates and returns a copy of this object.
	 * @return A duplicate copy of this object is created
	 */
	public Object clone()
	{
		String[] clonedntypes = new String[notifTypes.length];

		for(int i=0;i<notifTypes.length;i++)
			clonedntypes[i] = notifTypes[i];

		return new MBeanNotificationInfo(clonedntypes, getName(), getDescription());
	}

	/**
	 * Returns the notification type string of the notification (in dot notation).
	 *
	 * @return A String array of the notification types are returned
	 */
	public String[] getNotifTypes()
	{
		return notifTypes;
	}

	/**
	 * Returns a human readable version of the MBeanNotificationInfo instance
	 * @return MBeanNotificationInfo of String type is returned
	 */
	public String toString()
	{
		return (super.toString() +
				"\n NotifTypes = "+ getNotifTypesString() );
	}

	//------------------------- Private methods ---------------------------//

	private String getNotifTypesString()
	{
		if(notifTypes == null || notifTypes.length == 0)
			return "null";

		String toRet = "";


		for(int i=0;i<notifTypes.length;i++)
		{
			if(i == notifTypes.length-1)
				toRet += notifTypes[i];
			else
				toRet += notifTypes[i] + ", ";
		}

		return toRet;
	}

	private void readObject(ObjectInputStream objectinputstream)
			        			throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        notifTypes=(String[])getfield.get("types", "");
	    }
	    catch(Exception exception) {}
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("types", notifTypes);
		objectoutputstream.writeFields();
	}
}