/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;

/**
 * Describes an argument of an operation or constructor exposed by a MBean.
 */
public class MBeanParameterInfo extends MBeanFeatureInfo
								implements Serializable, Cloneable
{
    /**
     * The type or class name of the data.
     */
	private String type = null;

	/* Serial version UID */
	private static final long serialVersionUID = 0x6725f5acd97b3602L;

	/**
     * Constructs a <CODE>MBeanParameterInfo</CODE> object.
     *
     * @param name The name of the data.
     *
     * @param type The type or class name of the data.
     *
     * @param description A human readable description of the data. Optional.
	 */
	public MBeanParameterInfo(String name, String type, String description)
	{
		super(name, description);
		this.type = type;
	}

	/**
	 * Returns the type or class name of the data.
	 *
	 * @return The class name is returned
	 */
	public String getType()
	{
		return type;
	}

	/**
	 * Creates and returns a copy of this object.
	 *
	 * @return A duplicate copy of this object is returned
	 */
	public Object clone()
	{
		return new MBeanParameterInfo(getName(), type, getDescription());
	}

	/**
	 * Returns a human readable version of the MBeanParameterInfo instance
	 *
	 * @return  human readable version of the MBeanParameterInfo instance is returned
	 */
	public String toString()
	{
		return (super.toString() +
				"\n Type = "+ type );
	}
}