/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * Should be implemented by an MBean that emits Notifications. It allows a
 * listener to be registered within the MBean as notification listener.
 */
public abstract interface NotificationBroadcaster
{
	/**
	 * Enables a couple (listener,handback) for a registered MBean to be added.
	 *
	 * @param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @param filter The filter object. If not specified, no filtering will be
	 * 				performed before handling notifications.
	 *
     * @param handback The context to be sent to the listener when a
	 * 				notification is emitted.
	 *
	 * @throws java.lang.IllegalArgumentException Listener parameter is null.
	 */
	public void addNotificationListener(NotificationListener listener,
										NotificationFilter filter,
										Object handback)
								 throws IllegalArgumentException;


	/**
	 * Enables a listener for an MBean to be removed. All couple
	 * (listener, handback) are removed.
	 *
	 * @param listener The listener object which will handles notifications
	 * 				emitted by the registered MBean.
	 *
	 * @throws ListenerNotFoundException The listener is not registered in the MBean.
	 */
	public void removeNotificationListener(NotificationListener listener)
                                throws ListenerNotFoundException;

	/**
	 * Returns a NotificationInfo object containing the name of the Java class
	 * of the notification and the notification types sent.
	 *
	 * @return This returns a array of MBeanNotificationInfo which
	 * 				contains the notification information
	 */
	public MBeanNotificationInfo[] getNotificationInfo();
}