/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * This class represents exceptions thrown in the MBeanServer when using the
 * java.lang.reflect classes to invoke methods on MBeans. It "wraps" the actual
 * java.lang.Exception thrown.
 */
public class ReflectionException extends JMException
{
    /**
     * The wrapped Exception
     */
    private Exception exception = null;

	/**
     * Creates a <CODE>ReflectionException</CODE> that wraps the actual
	 * <CODE>java.lang.Exception</CODE>.
	 *
	 * @param e Exception to be wraped by ReflectionException
	 */
	public ReflectionException(Exception e)
	{
		super();
		this.exception = e;
	}

	/**
     * Creates a <CODE>ReflectionException</CODE> that wraps the actual
	 * <CODE>java.lang.Exception</CODE> with a detail message.
	 *
	 * @param e Exception to be wraped by ReflectionException
	 *
	 * @param msg Detail message
	 */
    public ReflectionException(Exception e, String msg)
	{
		super(msg);
		exception = e;
	}

	/**
	*Returns the actual java.lang.Exception thrown
	* @return This returns the actual java.lang.Exception thrown
	*/
	public java.lang.Exception getTargetException(){
		return exception;
	}

	/**
	 * Overrides the printStackTrace method in java.lang.Throwable
	 */
	public void printStackTrace()
	{
		super.printStackTrace();

		if(exception != null)
			exception.printStackTrace();
	}

	/**
	 * Overrides the toString method in java.lang.Object
	 */
	public String toString()
	{
		if(exception == null || exception.getMessage() == null)
			return super.toString();

		return (super.toString() + "; nested exception is : " +
									exception.toString());
	}
}//End of class ReflectionException