/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * Represents strings that are arguments to relational constraints.
 * A <CODE>StringValueExp</CODE> may be used anywhere a <CODE>ValueExp</CODE>
 * is required.
 */
public class StringValueExp implements ValueExp
{
	private String val = null;

	private transient MBeanServer server = null;

	private StringValueExp v1 = null;

	private StringValueExp v2 = null;

	int type = -1;

	/**
	 * Constructs a StringValueExp with default val
	 */
	public StringValueExp()
	{
	}

	/**
     * Creates a new <CODE>StringValueExp</CODE> representing the string
	 * literal val.
	 *
	 * @param val Creates a StringValueExp representing the string literal val
	 */
	public StringValueExp(String val)
	{
		this.val = val;
	}

	//added for Query.plus(v1, v2)..
	StringValueExp(StringValueExp v1, StringValueExp v2, int type)
	{
		this.type = type;
		this.v1 = v1;
		this.v2 = v2;
	}

	/**
     * Returns the string represented by the <CODE>StringValueExp</CODE> instance.
     *
	 * @return This returns the string represented by the StringValueExp instance.
	 */
	public String getValue()
	{
		return val;
	}

	/**
     * Applies the ValueExp on a MBean. Implements the method in the ValueExp.
     *
     * @param name The name of the MBean on which the ValueExp will be applied.
     *
     * @return  The <CODE>ValueExp</CODE>.
	 *
	 * @exception BadStringOperationException  This exception is thrown when
	 * 				an invalid string operation is passed to a method for
	 * 				constructing a query.
	 *
	 * @exception BadBinaryOpValueExpException This exception is thrown when
	 * 				an invalid expression is passed to a method for
	 * 				constructing a query.
	 *
	 * @exception BadAttributeValueExpException The BadAttributeValueExpException
	 * 				is thrown when an invalid MBean attribute is passed
	 * 				to a query constructing method.
	 *
     * @exception InvalidApplicationException This exception is thrown when
	 * 				an attempt is made to apply either of the following:
	 * 				A subquery expression to an MBean
	 * 				A qualified attribute expression to an MBean of the wrong class
	 */
	public ValueExp apply(ObjectName name)
								throws  BadStringOperationException,
										BadBinaryOpValueExpException,
										BadAttributeValueExpException,
										InvalidApplicationException
	{
		return this;
	}

	/**
	 * Sets the MBeanServer on which the query is to be accessed.
	 *
	 * @param server - The MBeanServer on which the query is to be accessed
	 */
	public void setMBeanServer(MBeanServer server)
	{
		this.server = server;
	}

	/**
	 * Returns the string representing the object.
	 * @return Returns the string representation of object
	 */
	public String toString()
	{
		return val;
	}

	Object getObject()
	{
		if(type == Query.PLUS)
		{
		   return (String)v1.getObject() + (String)v2.getObject();
		}

		return val;
	}
}//End of class StringValueExp.