/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.loading;

import java.io.Serializable;
import java.util.Vector;

/**
 * Keeps the list of Class Loaders registered in the MBean Server. It provides
 * the necessary methods to load classes using the registered Class Loaders.
 */
public class DefaultLoaderRepository implements Serializable
{
	/** List of class loaders */
	protected static Vector loaders = new Vector();

	/**
	 * The default constructor
	 */
	public DefaultLoaderRepository()
	{
	}

	/**
	 * Go through the list of class loaders and try to load the requested class.
	 * The method will stop as soon as the class is found. If the class is
	 * not found the method will throw a ClassNotFoundException exception.
	 *
     * @param className The name of the class to be loaded.
     *
	 * @return This returns a Class after loading the requested class
	 *
     * @exception java.lang.ClassNotFoundException The specified class
     * 				could not be found.
	 */
	public static Class loadClass(String className)
                                 throws ClassNotFoundException
	{
		return loadClass((Vector)loaders.clone(), className);
	}

	/**
	 * Go through the list of class loaders but exclude the given class loader,
	 * then try to load the requested class. The method will stop as soon as
	 * the class is found. If the class is not found the method will throw a
	 * ClassNotFoundException exception.
	 *
	 * @param className The name of the class to be loaded.
	 *
	 * @param loader The class loader to be excluded.
	 *
	 * @return Returns a requested class which is loaded after goining through
	 * 				the list of class loaders but exclude the given class loader
	 *
	 * @exception java.lang.ClassNotFoundException The specified class
	 * 				could not be found.
	 */
	public static Class loadClassWithout(ClassLoader loader, String className)
                                        throws ClassNotFoundException
	{
		Vector validLoaders = (Vector)loaders.clone();
		int size = validLoaders.size();

		for(int i=0;i<size;i++)
		{
			try
			{
				ClassLoader cl = (ClassLoader)loaders.elementAt(i);
				if(cl.equals(loader))
					continue;

				return cl.loadClass(className);
			}
			catch(ClassNotFoundException cnfe)
			{
				continue;
			}
		}

		throw new ClassNotFoundException(className);
	}

	//--------------------------- Package methods ---------------------------//

	/**
	 * Go through the list of class loaders and try to load the requested class.
	 * The method will stop as soon as the class is found. If the class is not
	 * found the method will throw a ClassNotFoundException exception.
	 *
     * @param className The name of the class to be loaded.
     *
	 * @return This returns a Class after loading the requested class
	 *
     * @exception java.lang.ClassNotFoundException The specified class
     * 				could not be found.
	 */
	static Class loadClass(Vector validLoaders, String className)
                                 throws ClassNotFoundException
	{
		int size = validLoaders.size();
		Class clazz = null;

		Vector tempLoaders = new Vector();
		Vector positions = new Vector();

		for(int i=0;i<validLoaders.size();i++)
		{
			try
			{
				ClassLoader cloader = (ClassLoader)validLoaders.elementAt(i);

				validLoaders.removeElementAt(i);
				tempLoaders.addElement(cloader);
				positions.addElement(new Integer(positions.size()+i));
				i--;

				if(cloader instanceof MLet)
				{
					((MLet)cloader).setUseDefaultLoader(true);
					clazz = cloader.loadClass(className);
					((MLet)cloader).setUseDefaultLoader(false);
				}
				else
					clazz = cloader.loadClass(className);

				break;

			}
			catch(ClassNotFoundException cnfe)
			{
				// continue;
			}
		}

		if(clazz != null)
			return clazz;

		throw new ClassNotFoundException(className);
	}

	static Class loadClassFromDefaultLoader(String className)
                                 	throws ClassNotFoundException
	{
		int size = loaders.size();
		Class clazz = null;

		for(int i=0; i<loaders.size(); i++)
		{
			try
			{
				ClassLoader cloader = (ClassLoader)loaders.elementAt(i);

				if(cloader instanceof MLet)
					continue;

				clazz = cloader.loadClass(className);
				break;

			}
			catch(ClassNotFoundException cnfe)
			{
				// continue;
			}
		}

		if(clazz != null)
			return clazz;

		throw new ClassNotFoundException(className);
	}
}