/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.loading;

import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Set;

import javax.management.ServiceNotFoundException;

/**
 * This interface exposes the remote management interface of the MLet MBean.
 */
public abstract interface MLetMBean
{
	/**
	 * Appends the specified URL to the list of URLs to search for classes and
	 * resources.
	 *
	 * @param url This url will be appended with the list of URLs to search
	 * 				for classes and resources
	 *
	 * @exception javax.management.ServiceNotFoundException The specified
	 * 				URL is malformed.
	 */
	public void addURL(String url) throws ServiceNotFoundException;

	/**
	 * Appends the specified URL to the list of URLs to search for classes and
	 * resources.
	 *
	 * @param url the url to be added to this MLetMBean
	 */
	public void addURL(URL url);


	/**
	 * Gets the current directory used by the library loader for storing
	 * native libraries before they are loaded into memory.
	 *
	 * @return The current directory used by the library loader.
	 */
	public String getLibraryDirectory();


	/**
	 * Loads a text file containing MLET tags that define the MBeans to be
	 * added to the agent. The location of the text file is specified by a URL.
	 * The MBeans specified in the MLET file will be instantiated and
	 * registered by the MBeanServer.
	 *
	 * @param url  The URL of the text file to be loaded as String object.
	 *
	 * @return A set containing one entry per MLET tag in the m-let text
	 * 				file loaded. Each entry specifies either the ObjectInstance
	 * 				for the created MBean, or a throwable object (that is, an
	 * 				error or an exception) if the MBean could not be created.
	 *
	 * @exception javax.management.ServiceNotFoundException  One of the
	 * 				following errors has occurred:
	 * 				The m-let text file does not contain an MLET tag,
	 * 				the m-let text file is not found,
	 * 				a mandatory attribute of the MLET tag is not specified,
	 * 				the value of url is malformed.
	 */
	public Set getMBeansFromURL(String url) throws ServiceNotFoundException;


	/**
	 * Loads a text file containing MLET tags that define the MBeans to be
	 * added to the agent. The location of the text file is specified by a URL.
	 * The MBeans specified in the MLET file will be instantiated and
	 * registered by the MBeanServer.
	 *
     * @param url  The URL of the text file to be loaded as URL object.
     *
     * @return A set containing one entry per MLET tag in the m-let text
	 * 				file loaded. Each entry specifies either the ObjectInstance
	 * 				for the created MBean, or a throwable object (that is, an
	 * 				error or an exception) if the MBean could not be created.
     *
     * @exception javax.management.ServiceNotFoundException - One of the
	 * 				following errors has occurred:
	 * 				The m-let text file does not contain an MLET tag,
	 * 				the m-let text file is not found,
	 * 				a mandatory attribute of the MLET tag is not specified,
	 * 				the value of url is null.
	 */
	public Set getMBeansFromURL(java.net.URL url) throws ServiceNotFoundException;


	/**
	 * Finds the resource with the given name. A resource is some data
	 * (images, audio, text, etc) that can be accessed by class code in a
	 * way that is independent of the location of the code.The name of a
	 * resource is a "/"-separated path name that identifies the resource.
	 *
	 * @param name The resource name
	 *
	 * @return An URL for reading the resource, or null if the resource
	 * 				could not be found or the caller doesn't have adequate
	 * 				privileges to get the resource.
	 */
	public URL getResource(String name);


	/**
	 * Returns an input stream for reading the specified resource. The
	 * search order is described in the documentation for getResource(String).
	 *
	 * @param name The resource name
	 *
	 * @return An input stream for reading the resource, or null if the
	 * 				resource could not be found
	 */
	public InputStream getResourceAsStream(String name);


	/**
	 * Finds all the resources with the given name. A resource is some data
	 * (images, audio, text, etc) that can be accessed by class code in a way
	 * that is independent of the location of the code. The name of a
	 * resource is a "/"-separated path name that identifies the resource.
	 *
	 * @param name The resource name
	 *
	 * @return An enumeration of URL to the resource. If no resources could
	 * 				be found, the enumeration will be empty. Resources that
	 * 				the doesn't have access to will not be in the enumeration.
	 *
	 * @exception IOException - Signals that an I/O exception of some sort
	 * 				has occurred. This class is the general class of exceptions
	 * 				produced by failed or interrupted I/O operations.
	 */
	public Enumeration getResources(String name) throws IOException;

	/**
	 * Returns the search path of URLs for loading classes and resources.
	 * This includes the original list of URLs specified to the constructor,
	 * along with any URLs subsequently appended by the addURL() method.
	 *
	 * @return This returns the search path of URLs for loading classes and resources
	 */
	public URL[] getURLs();

	/**
	 * Sets the directory used by the library loader for storing native
	 * libraries before they are loaded into memory.
	 *
	 * @param libdir The directory used by the library loader.
	 */
	public void setLibraryDirectory(java.lang.String libdir);

}//End of interface MLetMBean