/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.loading;

import java.lang.reflect.Array;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.io.IOException;
import java.io.StreamCorruptedException;


class MLetObjectInputStream extends ObjectInputStream
{

    public MLetObjectInputStream(InputStream inputstream, MLet mlet)
        throws IOException, StreamCorruptedException
    {
        super(inputstream);
        if(mlet == null)
        {
            throw new IllegalArgumentException("Illegal null argument to MLetObjectInputStream");
        } else
        {
            loader = mlet;
            return;
        }
    }

    private Class primitiveType(char c)
    {
        switch(c)
        {
        case 66: // 'B'
            return Byte.TYPE;

        case 67: // 'C'
            return Character.TYPE;

        case 68: // 'D'
            return Double.TYPE;

        case 70: // 'F'
            return Float.TYPE;

        case 73: // 'I'
            return Integer.TYPE;

        case 74: // 'J'
            return Long.TYPE;

        case 83: // 'S'
            return Short.TYPE;

        case 90: // 'Z'
            return Boolean.TYPE;

        case 69: // 'E'
        case 71: // 'G'
        case 72: // 'H'
        case 75: // 'K'
        case 76: // 'L'
        case 77: // 'M'
        case 78: // 'N'
        case 79: // 'O'
        case 80: // 'P'
        case 81: // 'Q'
        case 82: // 'R'
        case 84: // 'T'
        case 85: // 'U'
        case 86: // 'V'
        case 87: // 'W'
        case 88: // 'X'
        case 89: // 'Y'
        default:
            return null;
        }
    }

    protected Class resolveClass(ObjectStreamClass objectstreamclass)
        throws IOException, ClassNotFoundException
    {
        String s = objectstreamclass.getName();
        if(s.startsWith("["))
        {
            int i;
            for(i = 1; s.charAt(i) == '['; i++);
            Class class1;
            if(s.charAt(i) == 'L')
            {
                class1 = loader.loadClass(s.substring(i + 1, s.length() - 1));
            } else
            {
                if(s.length() != i + 1)
                    throw new ClassNotFoundException(s);
                class1 = primitiveType(s.charAt(i));
            }
            int ai[] = new int[i];
            for(int j = 0; j < i; j++)
                ai[j] = 0;

            return Array.newInstance(class1, ai).getClass();
        } else
        {
            return loader.loadClass(s);
        }
    }

    public ClassLoader getClassLoader()
    {
        return loader;
    }

    private MLet loader;
}