/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.loading;

import java.io.BufferedReader;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.Vector;
import java.util.StringTokenizer;
import java.text.ParseException;

/**
 * This class does the parsing of management applet(MLET) downloaded from
 * remote host. Makes a mLetVector which contains SimpleMLet objects, which
 * describes the information derived from the MLET file for each remote mbean.
 */
class MLetParser
{
	StreamTokenizer strTok = null;
	String fileString = "";
	Vector mLetVector = null;

	MLetParser(String fileName) throws ParseException
	{
		try
		{
			Reader r = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
			strTok = new StreamTokenizer(r);

			initTokens();

			mLetVector = new Vector();

			parseMLet();
		}
		catch(Exception e)
		{
			throw (ParseException)e;
		}
	}

	MLetParser(Reader r) throws ParseException
	{
		try
		{
			strTok = new StreamTokenizer(r);

			initTokens();

			mLetVector = new Vector();

			parseMLet();
		}
		catch(Exception e)
		{
			throw (ParseException)e;
		}
	}

	Vector getMLetVector()
	{
		return mLetVector;
	}

	void initTokens()
	{
		strTok.wordChars('{','}');
		strTok.wordChars('(',')');
		strTok.wordChars('0','9');
		strTok.wordChars(',',',');

		//strTok.wordChars(' ',' ');
		//strTok.wordChars('\t','\t');

		strTok.wordChars('<','<');
		strTok.wordChars('>','>');
		//strTok.wordChars('>','>');
		//strTok.ordinaryChar('<');
		//strTok.ordinaryChar('>');
		//strTok.ordinaryChar('=');
		strTok.wordChars('/','/');

		strTok.parseNumbers();
		strTok.eolIsSignificant(false);
		//tok.ordinaryChars('\0','\0');

		//strTok.wordChars('#','z');
		strTok.wordChars('"','"');
		strTok.wordChars('_','_');
		strTok.wordChars(':',':');
		strTok.ordinaryChar(';');
		strTok.ordinaryChar('_');
		strTok.wordChars('@','@');
		strTok.ordinaryChar('(');
		strTok.ordinaryChar(')');
		//strTok.ordinaryChar('.');
		//strTok.ordinaryChar(',');
	}

	private boolean mletFlag = false;
	private boolean tagClosed = false;
	private SimpleMLet mLet = null;

	private void parseMLet() throws Exception
	{
		while(strTok.ttype != strTok.TT_EOF)
		{
			if(strTok.sval != null)
			{
				String tokVal =  strTok.sval;

				fileString +=tokVal+"\n";

				if(tokVal.equals("<MLET") )
				{
					mLet = new SimpleMLet();

					if(mletFlag)
						throw new ParseException("Parse Error", strTok.lineno());
					mletFlag = true;
					tagClosed = false;

					strTok.sval = "";
					strTok.nextToken();
					try{

					while( strTok.ttype != strTok.TT_EOF
						&& ( strTok.sval!=null && !strTok.sval.equals("<MLET")))
					{
						//if(strTok.sval == null)
						//{
						//	strTok.nextToken();
						//	continue;
						//}
						tokVal = strTok.sval.trim();

						fileString +=tokVal+"\n";

						if(tokVal.equals(">"))
						{
							tagClosed = true;
						}
						else if(tokVal.equals("CODE"))
						{
							strTok.nextToken();
							int type = strTok.ttype;
							if(type != 61)
								throw new ParseException("Parse Error", strTok.lineno());

							strTok.nextToken();
							tokVal = strTok.sval.trim();
							fileString +=tokVal+"\n";


							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}

							if(tokVal.endsWith("class"))
							{
								tokVal = tokVal.substring(0,tokVal.length()-6);
							}
							tokVal = tokVal.replace('@','_');

							mLet.setCode(tokVal);
						}
						else if(tokVal.equals("OBJECT"))
						{
							strTok.nextToken();
							int type = strTok.ttype;
							if(type != 61)
								throw new ParseException("Parse Error", strTok.lineno());

							strTok.nextToken();
							tokVal = strTok.sval.trim();
							fileString +=tokVal+"\n";

							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}
							tokVal = tokVal.replace('@','_');

							mLet.setObject(tokVal);
						}
						else if(tokVal.equals("ARCHIVE"))
						{
							strTok.nextToken();
							int type = strTok.ttype;
							if(type != 61)
							{
								throw new ParseException("Parse Error", strTok.lineno());
							}

							strTok.nextToken();
							tokVal = strTok.sval.trim();

							fileString +=tokVal+"\n";
							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}
							else
							{
								throw new ParseException("Invalid Archive Format", strTok.lineno());
							}

							Vector vec = parseArchiveTok(tokVal);
							mLet.setArchive(vec);
						}
						else if(tokVal.equals("CODEBASE"))
						{
							strTok.nextToken();
							int type = strTok.ttype;
							if(type != 61)
								throw new ParseException("Parse Error", strTok.lineno());

							strTok.nextToken();
							tokVal = strTok.sval.trim();
							fileString +=tokVal+"\n";

							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}
							tokVal = tokVal.replace('@','_');

							mLet.setCodebase(tokVal);
						}
						else if(tokVal.equals("NAME"))
						{
							strTok.nextToken();

							int type = strTok.ttype;
							if(type != 61)
								throw new ParseException("Parse Error", strTok.lineno());

							strTok.wordChars('=','=');

							strTok.nextToken();
							tokVal = strTok.sval.trim();
							fileString +=tokVal+"\n";
							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}
							tokVal = tokVal.replace('@','_');

							mLet.setName(tokVal);
							strTok.ordinaryChar('=');
						}
						else if(tokVal.equals("VERSION"))
						{
							strTok.nextToken();
							int type = strTok.ttype;
							if(type != 61)
								throw new ParseException("Parse Error", strTok.lineno());

							strTok.nextToken();
							tokVal = strTok.sval.trim();
							fileString +=tokVal+"\n";

							if(tokVal.indexOf("-") != -1)
								throw new ParseException("Version number cannot be negative", strTok.lineno());
							if(tokVal.startsWith("\""))
							{
								tokVal = tokVal.substring(1,tokVal.length()-1);
							}
							tokVal = tokVal.replace('@','_');

							mLet.setVersion(tokVal);
						}
						else if(tokVal.equals("<ARG"))
						{
							strTok.nextToken();
							parseArglist(mLet, strTok);
							//mLet.setArglist(hashTab);
						}
						else if(strTok.sval.equals("</MLET>"))
						{
							if(!tagClosed)
								throw new ParseException("MLetTag not closed", strTok.lineno());
							mletFlag = false;
						}
						strTok.nextToken();
					}
				}
				catch(Exception e)
				{
					throw new ParseException(e.getMessage(), strTok.lineno());
				}
					strTok.pushBack();

					if(mLet.getCode() == null && mLet.getObject() == null)
						throw new ParseException("No Code/Object defined", strTok.lineno());

					if(mLet.getArchive() == null)
						throw new ParseException("Parse Error",strTok.lineno());

					mLetVector.addElement(mLet);
				}
				else
				{
					strTok.nextToken();
				}
			}
			else
			{
				strTok.nextToken();
			}
		}
	}

	private Vector parseArchiveTok(String token) throws Exception
	{
		if(token == null)
		{
			throw new Exception("Invalid ArchiveList ");
		}

		Vector toRet = new Vector();

		token = token.replace('@','_');

		if(token.indexOf(",") != -1)
		{
			//More than One jar file is present.so following checks have to be performed.
			//1.Each file must be separated by a comma
			//2.The whole list must be enclosed in "".

			StringTokenizer st = new StringTokenizer(token, ",");
	        while (st.hasMoreTokens())
	        {
				toRet.addElement(st.nextToken());
			}
		}
		else if(token.indexOf(" ") != -1)
			throw new Exception("Invalid Archive Tag");
		else
			toRet.addElement(token);

		return toRet;
	}

	private void parseArglist(SimpleMLet entry, StreamTokenizer strTok) throws Exception
	{
		//boolean argTagClosed = false;
		//Hashtable argTable = entry.getArglist();
		ArrayList argTypes = entry.getArgTypes();
		ArrayList argValues = entry.getArgValues();

		strTok.ordinaryChar('>');

		//int type = strTok.ttype;
		//if(type != 61)
		//	throw new ParseException("Parse Error", strTok.lineno());
		String argtype = null;
		while(strTok.ttype != 62)
		{
			String tokVal = strTok.sval.trim();

			fileString +=tokVal+"\n";

			if(tokVal.equals("TYPE"))
			{
				strTok.nextToken();
				int type = strTok.ttype;
				if(type != 61)
					throw new ParseException("Parse Error", strTok.lineno());

				strTok.nextToken();
				tokVal = strTok.sval.trim();
				fileString +=tokVal+"\n";
				if(tokVal.startsWith("\""))
				{
					tokVal = tokVal.substring(1,tokVal.length()-1);
				}
				tokVal = tokVal.replace('@','_');


				argtype = tokVal;
			}
			else if(tokVal.equals("VALUE"))
			{
				strTok.nextToken();
				int type = strTok.ttype;
				if(type != 61)
					throw new ParseException("Parse Error", strTok.lineno());

				strTok.nextToken();


				if(strTok.ttype == strTok.TT_NUMBER)
				{
					String temp = "" + strTok.nval;
					if(!temp.endsWith(".0"))
						tokVal = temp;
					else
							tokVal = "" + (int)strTok.nval;
				}
				else
					tokVal = strTok.sval.trim();

				//tokVal = strTok.sval.trim();

				fileString +=tokVal+"\n";

				if(argtype == null)
					new Exception("Wrong argList definition line no: "+strTok.lineno());

				//argTable.put(argtype, tokVal);

				if(argtype.startsWith("\""))
				{
					argtype = argtype.substring(1,argtype.length()-1);
				}
				argtype.replace('@','_');
				argTypes.add(argtype);
				if(tokVal.startsWith("\""))
				{
					tokVal = tokVal.substring(1,tokVal.length()-1);
				}
				tokVal = tokVal.replace('@','_');

				Object val = returnDefaultValue(tokVal, argtype);
				argValues.add(val);
				argtype = null;
			}
			else
			{
				throw new ParseException("Parse Error", strTok.lineno());
			}
			strTok.nextToken();
		}

		strTok.wordChars('>','>');
	}

	private Object returnDefaultValue(String value, String type)
		throws Exception
	{
		if(type.indexOf("String") != -1)
			return value;
		if(type.indexOf("Integer") != -1 || type.indexOf("int") != -1 )
			return new Integer(value);
		if(type.indexOf("Long") != -1 || type.indexOf("long") != -1 )
			return new Long(value);
		if(type.indexOf("Boolean") != -1 || type.indexOf("boolean") != -1 )
			return new Boolean(value);
		if(type.indexOf("Float") != -1 || type.indexOf("float") != -1 )
			return new Float(value);
		if(type.indexOf("Double") != -1 || type.indexOf("double") != -1 )
		{
			return new Double(value);
		}
		if(type.indexOf("Byte") != -1 || type.indexOf("byte") != -1 )
			return new Byte(value);
		if(type.indexOf("Character") != -1 || type.indexOf("char") != -1 )
			return new Character(value.charAt(0));
		if(type.indexOf("Short") != -1 || type.indexOf("short") != -1 )
			return new Short(value);
		return null;
	}
}