/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import javax.management.DynamicMBean;
import javax.management.PersistentMBean;
import javax.management.MBeanException;
import javax.management.InstanceNotFoundException;
import javax.management.RuntimeOperationsException;

/**
 * ModelMBean interface is used to expose its management data,
 * operations and notifications for use by the management system.
 * This interface provides a management template for managed resources.
 * The adaptors will interact with the ModelMBean through the JMX agent(server).
 * It will find the managed resource provided attributes and operations through
 * the MBeanInfo object. It find the behavior details about the supported
 * attribute, operations and notifications through this ModelMBean interface.
 * <p>
 * This interface must be implemented by the ModelMBeans.
 * An implementation of this interface must be shipped with every JMX Agent.
 * <p>
 * Java resources wishing to be manageable instatiate the ModelMBean using
 * the MBeanServer's createMBean method. The resource then sets the
 * ModelMBeanInfo (with Descriptors) for the ModelMBean instance. The
 * attributes and operations exposed via the ModelMBeanInfo for the ModelMBean
 * are accessible from Mbeans, connectors/adapters like other MBeans. Through
 * the ModelMBeanInfo Descriptors, values and methods in the managed
 * application can be defined and mapped to attributes and operations of the
 * ModelMBean. This mapping can be defined during development in an XML
 * formatted file or dynamically and programmatically at runtime.
 * <p>
 * Every ModelMBean which is instantiated in the MBeanServer becomes manageable:
 * its attributes and operations become remotely accessible through the
 * connectors/adaptors connected to that MBeanServer. A Java object cannot be
 * registered in the MBeanServer unless it is a JMX compliant MBean. By
 * instantiating a ModelMBean, resources are guaranteed that the MBean is valid.
 * <p>
 * MBeanException and RuntimeOperatiosException must be thrown on every public
 * method. This allows for wrappering exceptions from distributed communications
 * (RMI, EJB, etc.). These exceptions do not have to be thrown by the implementation
 */
public interface ModelMBean extends DynamicMBean,
									PersistentMBean,
									ModelMBeanNotificationBroadcaster
{
	/**
	 * Initializes a ModelMBean object using ModelMBeanInfo passed in. The
	 * ModelMBean must be instantiated, but not registered with the MBeanServer.
	 * After the ModelMBean's ModelMBeanInfo (with Descriptors) are customized,
	 * the ModelMBean should be registered with the MBeanServer.
	 *
	 * @param inModelMBeanInfo The ModelMBeanInfo object to be used by the ModelMBean.
	 *
	 * @exception MBeanException The constructor of the MBeanInfo has
	 * 				return null or thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The MBeanInfo passed in parameter is null or invalid.
	 */
	public void setModelMBeanInfo(ModelMBeanInfo inModelMBeanInfo)
								throws MBeanException,
									   RuntimeOperationsException;


	/**
	 * Sets the instance handle of the object against which to execute all
	 * methods in this ModelMBean management interface (MBeanInfo and
	 * Descriptors).
	 *
	 * @param mr Object that is the managed resource
	 *
	 * @param mr_type The type of reference for the managed resource.
	 * 				Can be: ObjectReference, Handle, IOR, EJBHandle, RMIReference.
	 * 				If the MBeanServer cannot process the mr_type passed in, an
	 * 				InvalidTargetTypeException will be thrown.
	 *
	 * @exception MBeanException The initializer of the object has thrown an exception.
	 *
	 * @exception RuntimeOperationsException Wraps an IllegalArgumentException:
	 * 				The managed resource or managed resoure type passed in
	 * 				parameter is null or invalid.
	 *
	 * @exception InstanceNotFoundException The managed resource object could
	 * 				not be found
	 *
	 * @exception InvalidTargetObjectTypeException The managed resource type
	 * 				cannot be processed by the ModelMBean or JMX Agent.
	 */
	public void setManagedResource(Object mr, String mr_type)
									throws MBeanException,
										   RuntimeOperationsException,
										   InstanceNotFoundException,
										   InvalidTargetObjectTypeException;
}