/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;


/**
 * Exposes the remote management interface of the string monitor MBean.
 */
public abstract interface StringMonitorMBean extends MonitorMBean
{
	/**
	 * This method gets the value of the derived gauge. The derived gauge is
	 * either the exact value of the observed attribute, or the difference
	 * between the two consecutive observed values of the attribute.
	 *
	 * @return An instance of java.lang.String giving the value of the derived gauge.
	 */
	public String getDerivedGauge();

	/**
	 * This method gets the value of the derived gauge time stamp. The derived
	 * gauge time stamp is the value(in the nearest miliseconds) when the
	 * notification was triggered.
	 *
	 * @return long value representing the time the notification was triggered.
	 */
	public long getDerivedGaugeTimeStamp();

	/**
	 * This method sets the differing notification's on/off switch value.
	 *
	 * @return  The differing notification's on/off switch value.
	 */
	public boolean getNotifyDiffer();

	/**
	 * This method sets the differing notification's on/off switch value.
	 *
	 * @param value - The differing notification's on/off switch value.
	 */
	public void setNotifyDiffer(boolean value);

	/**
	 * This method gets the matching notification's on/off switch value.
	 *
	 * @return The matching notification's on/off switch value.
	 */

	public boolean getNotifyMatch();

	/**
	 * This method sets the matching notification's on/off switch value.
	 *
	 * @param value - The matching notification's on/off switch value.
	 */
	public void setNotifyMatch(boolean value);

	/**
	 * This method gets the string to compare with the observed attribute.
	 *
	 * @return the String to be compared.
	 */
	public String getStringToCompare();

	/**
	 * This method sets the string to compare with the observed attribute.
	 *
	 * @param value The String Value to be compared.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified string
	 * 				to compare is null.
	 */
	public void setStringToCompare(String value) throws IllegalArgumentException;
}