/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import javax.management.*;

import java.util.*;

/**
 * The Relation Service is in charge of creating and deleting relation types
 * and relations, of handling the consistency and of providing query mechanisms.
 * <P>
 * It implements the NotificationBroadcaster by extending
 * NotificationBroadcasterSupport to send notifications when a relation
 * is removed from it.
 * <P>
 * It implements the NotificationListener interface to be able to receive
 * notifications concerning unregistration of MBeans referenced in relation
 * roles and of relation MBeans.  It implements the MBeanRegistration interface
 * to be able to retrieve its ObjectName and MBean Server.
 */
public class RelationService extends NotificationBroadcasterSupport
    implements RelationServiceMBean, MBeanRegistration, NotificationListener
{
	/** **/
    private HashMap relationsTable = new HashMap();

    /** **/
    private HashMap relationTypesTable = new HashMap();

	/** **/
    private HashMap relationObjNamesTable = new HashMap();

    /** **/
    private HashMap relationTypeObjTable = new HashMap();

	/** **/
    private HashMap relTypeToIDTable = new HashMap();

    private String ERR_VALUE = new String();

    private HashMap refMBeansTable = new HashMap();

    private boolean purgeFlag = true;

    private long notifNo = 0;

    private ObjectName relObjectName = null;

    private ObjectName objectName = null;

    private MBeanServer mbeanServer = null;

    private MBeanServerNotificationFilter notifFilter = null;

    private ArrayList notifList = new ArrayList();

    /**
     * Constructor initiates relation service
     * @param theImmediatePurgeFlg - flag to indicate when a notification is
	 * 				received for the unregistration of a MBean referenced in
	 * 				a relation, if an immediate "purge" of the relations (look
	 * 				for the relations no longer valid) has to be performed ,
	 * 				or if that will be performed only when the purgeRelations
	 * 				method will be explicitly called. true is immediate purge.
     */
    public RelationService(boolean theImmediatePurgeFlg)
	{
		this.purgeFlag = theImmediatePurgeFlg;
        try
        {
			objectName = new ObjectName("Services:type=RelationService");
		}
		catch(Exception e)
		{
		}
	}

    /**
     * Description copied from interface: MBeanRegistration
     * <P>
     * Allows the MBean to perform any operations needed after having been
	 * de-registered in the MBean mbeanServer.
	 * <P>
     * Specified by:
     *  			postDeregister in interface MBeanRegistration
     */
    public void postDeregister()
	{
 		//This is only for satisfying MBeanRegistration Interface.
    }

    /**
     * Description copied from interface: MBeanRegistration
     * <P>Allows the MBean to perform any operations needed after having been
	 * registered in the MBean server or after the registration has failed.
	 * <P>
     * Specified by:
     * 				postRegister in interface MBeanRegistration
     *    			Tags copied from interface: MBeanRegistration
     *
     * @param registrationDone - Indicates whether or not the MBean has been
	 * 				successfully registered in the MBean mbeanServer. The
	 * 				value false means that the registration phase has failed.
     */
	public void postRegister(java.lang.Boolean registrationDone)
	{
		// after registration with the MBean Server , register the
		// RelationService for MBeanServer Notifications.To register for
		// MBeanServer notifications, register with the Standard
		// MBeanServerDelegate ObjectName.
		try
		{
			if(registrationDone.booleanValue())
			{
				//This filter will be created with all the ObjectNames selected by default.
				mbeanServer.addNotificationListener(
					new ObjectName("JMImplementation:type=MBeanServerDelegate"),
					this, new MBeanServerNotificationFilter(), (new Integer(2)));
			}
		}
		catch(Exception e)
		{
			System.out.println("Exception occured while adding the RelationService as a Notification"+
								"Listener to the MBean Server for MBeanServerNotifications!!!!!!!");
        }
    }

    /**
     * Description copied from interface: MBeanRegistration
     * <P>Allows the MBean to perform any operations it needs before being
	 * de-registered by the MBean mbeanServer.
	 * <P>
     * Specified by:
     * 				preDeregister in interface MBeanRegistration
     *      		Tags copied from interface: MBeanRegistration
     *
     * @exception java.langException - This exception should be caught by the
	 * 				MBean server and re-thrown as an MBeanRegistrationException.
     */
	public void preDeregister()throws Exception
	{
		//This method is only to satisfy MBeanRegistration Interface ...Maybe used later
    }

    /**
     * Description copied from interface: MBeanRegistration
     * <P>
     * Allows the MBean to perform any operations it needs before being
	 * registered in the MBean mbeanServer. If the name of the MBean is not
	 * specified, the MBean can provide a name for its registration.
	 * <P>If any exception is raised, the MBean will not be registered in the
	 * MBean mbeanServer.
	 * <P>
     * Specified by:
     * 				preRegister in interface MBeanRegistration
     *      		Tags copied from interface: MBeanRegistration
     *
     * @param server - The MBean server in which the MBean will be registered.
     *
     * @param name - The object name of the MBean.
     *
     * @return The name of the MBean registered.
     *
     * @exception java.lang.Exception - This exception should be caught by the
	 * 				MBean server and re-thrown as an MBeanRegistrationException.
     */
    public ObjectName preRegister(MBeanServer server,ObjectName name)
		throws java.lang.Exception
	{
	    //store the reference of the MBeanServer and the ObjectName of this RelationService.
        this.mbeanServer = server;
        this.objectName = name;

        return name;
    }

    /**
     * Adds a MBean created by the user (and registered by him in the
	 * MBean Server) as a relation in the Relation Service.
     * <P>To be added as a relation, the MBean must conform to the following:
     * <P>- implement the Relation interface
     * <P>- have for RelationService ObjectName the ObjectName of current Relation Service
     * <P>- have a relation id unique and unused in current Relation Service
     * <P>- have for relation type a relation type created in the Relation Service
     * <P>- have roles conforming to the role info provided in the relation type.
     *
     * @param theRelObjectName - ObjectName of the relation MBean to be added.
     *
     * @throws java.lang.IllegalArgumentException - if null parameter
     *
     * @throws RelationServiceNotRegisteredException - if the Relation Service
	 * 				is not registered in the MBean Server
	 *
     * @throws java.lang.NoSuchMethodException - If the MBean does not
	 * 				implement the Relation interface
	 *
     * @throws InvalidRelationIdException - if:
     * <P>          - no relation identifier in MBean
     * <P>          - the relation identifier is already used in the Relation Service
     *
     * @throws InstanceNotFoundException - if the MBean for given ObjectName
	 * 				has not been registered
	 *
     * @throws InvalidRelationServiceException - if:
     * <P>          - no Relation Service name in MBean
     * <P>          - the Relation Service name in the MBean is not the one
	 * 					of the current Relation Service
     *
     * @throws RelationTypeNotFoundException - if:
     * <P>          - no relation type name in MBean
     * <P>          - the relation type name in MBean does not correspond to
	 * 					a relation type created in the Relation Service
	 *
     * @throws InvalidRoleValueException - if:
     * <P>          - the number of referenced MBeans in a role is less than
	 * 					expected minimum degree
     * <P>          - the number of referenced MBeans in a role exceeds
	 * 					expected maximum degree
     * <P>          - one referenced MBean in the value is not an Object of
	 * 					the MBean class expected for that role
     * <P>          - a MBean provided for a role does not exist
     *
     * @throws RoleNotFoundException - if a value is provided for a role that
	 * 				does not exist in the relation type
     */
    public void addRelation(ObjectName theRelObjectName)
							throws IllegalArgumentException,
							       RelationServiceNotRegisteredException,
							       NoSuchMethodException,
							       InvalidRelationIdException,
							       InstanceNotFoundException,
							       InvalidRelationServiceException,
							       RelationTypeNotFoundException,
							       RoleNotFoundException,
							       InvalidRoleValueException
	{
        if(theRelObjectName == null)
            throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

        if( ! (mbeanServer.isRegistered(this.objectName)))
           throw new RelationServiceNotRegisteredException(RelationErrs.ERR_REL_SERV_NOT_FOUND);

        if(! (mbeanServer.isRegistered(theRelObjectName)))
		{
            throw new InstanceNotFoundException();
        }

        //check if the given ObjectName is actually an instanceOf Relation.
		boolean isRelation = mbeanServer.isInstanceOf(theRelObjectName, "javax.management.relation.Relation");

        if(! isRelation)
            throw new NoSuchMethodException();

		Object[] params = new Object[0];
		String[] signature = new String[0];
		String relId = null;

		try
		{
			relId = (String)(mbeanServer.invoke(theRelObjectName,
										"getRelationId", params, signature));
		}
		catch(MBeanException me)
		{
			throw new RuntimeException((me.getTargetException()).getMessage());
		}
		catch (ReflectionException re)
		{
			throw new RuntimeException(re.getMessage());
		}

		if (relId == null)
		{
			throw new InvalidRelationIdException("Null Value");
		}

		//Now check if there is already exisitng RelationId.
		if(this.isRelationIdAlreadyExists(relId))
		{
			throw new InvalidRelationIdException(RelationErrs.ERR_RELATIONID_ALREADY_EXISTS);
		}

		ObjectName relServObjName = null;

		try
		{
			relServObjName = (ObjectName)(mbeanServer.invoke(theRelObjectName,
									"getRelationServiceName",params,signature));
		}
		catch (MBeanException me)
		{
			throw new RuntimeException((me.getTargetException()).getMessage());
		}
		catch (ReflectionException re)
		{
			throw new RuntimeException(re.getMessage());
		}

		boolean badRelServFlg = false;
		if (relServObjName == null)
		{
			badRelServFlg = true;
		}
		else if (!(relServObjName.equals(objectName)))
		{
			badRelServFlg = true;
		}

		if (badRelServFlg)
		{
			throw new InvalidRelationServiceException();
		}

		String relTypeName = null;
		try
		{
			relTypeName = (String)(mbeanServer.invoke(theRelObjectName,
									"getRelationTypeName",params,signature));
		}
		catch (MBeanException me)
		{
			throw new RuntimeException((me.getTargetException()).getMessage());
		}
		catch (ReflectionException re)
		{
			throw new RuntimeException(re.getMessage());
		}

		if (relTypeName == null)
		{
			throw new RelationTypeNotFoundException("RelationType is null");
		}

		RoleList roleList = null;

		try
		{
			roleList = (RoleList)(mbeanServer.invoke(theRelObjectName,
								"retrieveAllRoles",params,signature));
		}
		catch (MBeanException me)
		{
			throw new RuntimeException((me.getTargetException()).getMessage());
		}
		catch (ReflectionException re)
		{
			throw new RuntimeException(re.getMessage());
		}

		validateAndAddRelation(false, null, theRelObjectName, relId,
													relTypeName, roleList);

		relationObjNamesTable.put(theRelObjectName, relId);

		try
		{
			Object[] params1 = new Object[1];
			params1[0] = new Boolean(true);
			String[] sig1 = new String[1];
			sig1[0] = "java.lang.Boolean";
			mbeanServer.invoke(theRelObjectName,"setRelationServiceManagementFlag",params1,sig1);
		}
		catch (Exception exc) { }

		//TODO
		ArrayList newRefList = new ArrayList();
		newRefList.add(theRelObjectName);
		updateUnregistrationListener(newRefList, null);
    }

    /**
     * Adds given object as a relation type. The object is expected to
	 * implement the RelationType interface.
	 *
     * @param theRelTypeObj - relation type object (implementing the
	 * 				RelationType interface)
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception InvalidRelationTypeException - if there is already a
	 * 				relation type with that name.
     */
    public void addRelationType(RelationType theRelTypeObj)
				throws IllegalArgumentException, InvalidRelationTypeException
	{
		if (theRelTypeObj == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		List roleInfoList = theRelTypeObj.getRoleInfos();

		if (roleInfoList == null)
		{
			throw new InvalidRelationTypeException("Null Value");
		}

		RoleInfo[] roleInfoArray = new RoleInfo[roleInfoList.size()];
		int i = 0;

		Iterator iter = roleInfoList.iterator();
		while(iter.hasNext())
		{
			RoleInfo roleInfo = (RoleInfo)(iter.next());
			roleInfoArray[i] = roleInfo;
			i++;
		}

		checkRoleInfoConsistency(roleInfoArray);
		validateAndAddRelationType(theRelTypeObj);
     }

    /**
     * Checks if given Role can be read in a relation of the given type.
     *
     * @param theRoleName - name of role to be checked
     *
     * @param theRelTypeName - name of the relation type
     *
     * @return an Integer wrapping an integer corresponding to possible
	 * 				problems represented as constants in RoleUnresolved:
     * <P>   - 0 if role can be read
     * <P>   - integer corresponding to RoleStatus.NO_ROLE_WITH_NAME
     * <P>   - integer corresponding to RoleStatus.ROLE_NOT_READABLE
     *
     * @exception IllegalArgument - if null parameter
     *
     * @exception RelationTypeNotFoundException - if the relation type is
	 * 				not known in the Relation Service
     */
    public Integer checkRoleReading(String theRoleName, String theRelTypeName)
				throws IllegalArgumentException, RelationTypeNotFoundException
	{
		if (theRoleName == null || theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Integer toRet = null;
		RelationType relType = getRelationType(theRelTypeName);

		try
		{
			RoleInfo roleInfo = relType.getRoleInfo(theRoleName);

			toRet =  getRoleStatusInteger(theRoleName,null,roleInfo,false,1);

		}
		catch (RoleInfoNotFoundException e)
		{
			toRet = new Integer(RoleStatus.NO_ROLE_WITH_NAME);
		}

		return toRet;
    }

    /**
     * Checks if given Role can be set in a relation of given type.
     *
     * @param theRole - role to be checked
     *
     * @param theRelTypeName - name of relation type
     *
     * @param theInitFlg - flag to specify that the checking is done for the
	 * 				initialisation of a role, write access shall not be verified.
	 *
     * @return an Integer wrapping an integer corresponding to possible
	 * 				problems represented as constants in RoleUnresolved:
     *   - 0 if role can be set
     *   - integer corresponding to RoleStatus.NO_ROLE_WITH_NAME
     *   - integer for RoleStatus.ROLE_NOT_WRITABLE
     *   - integer for RoleStatus.LESS_THAN_MIN_ROLE_DEGREE
     *   - integer for RoleStatus.MORE_THAN_MAX_ROLE_DEGREE
     *   - integer for RoleStatus.REF_MBEAN_OF_INCORRECT_CLASS
     *   - integer for RoleStatus.REF_MBEAN_NOT_REGISTERED
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationTypeNotFoundException - if unknown relation type
     */
    public Integer checkRoleWriting(Role theRole, String theRelTypeName, Boolean theInitFlg)
				throws IllegalArgumentException, RelationTypeNotFoundException
	{
		if (theRole == null || theRelTypeName == null || theInitFlg == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = getRelationType(theRelTypeName);

		String roleName = theRole.getRoleName();
		ArrayList roleValue = (ArrayList)(theRole.getRoleValue());
		boolean writeFlag = true;

		if (theInitFlg.booleanValue())
		{
			writeFlag = false;
		}

		RoleInfo roleInfo = null;

		try
		{
			roleInfo = relType.getRoleInfo(roleName);
		}
		catch (RoleInfoNotFoundException exc)
		{
			return new Integer(RoleStatus.NO_ROLE_WITH_NAME);
		}

		Integer toRet = getRoleStatusInteger(roleName,roleValue,roleInfo,writeFlag,2);

		return toRet;
    }

    /**
     * Creates a simple relation (represented by a RelationSupport object) of
	 * given relation type, and adds it in the Relation Service.
     * <P>Roles are initialized according to the role list provided in
	 * parameter. The ones not initialised using that mean are set to an
	 * empty ArrayList of ObjectNames.
     * <P>A RelationNotification, with type RELATION_BASIC_CREATION, is sent.
     *
     * @param theRelId - relation identifier, to uniquely identify the
	 * 				relation inside the Relation Service
	 *
     * @param theRelTypeName - name of the relation type (has to be created
	 * 				in the Relation Service)
	 *
     * @param theRoleList - role list to initialise roles of the relation
	 * 				(can be null).
	 *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
     * @exception java.lang.IllegalArgumentException - if null paramater
     *
     * @exception RoleNotFoundException - if a value is provided for a role
	 * 				that does not exist in the relation type
	 *
     * @exception InvalidRelationIdException - if relation id already used
     *
     * @exception RelationTypeNotFoundException - if relation type not known
	 * 				in Relation Service
	 *
     * @exception InvalidRoleValueException - if:
     *      - the same role name is used for two different roles
     *      - the number of referenced MBeans in given value is less than
	 * 			expected minimum degree
     *      - the number of referenced MBeans in provided value exceeds
	 * 			expected maximum degree
     *      - one referenced MBean in the value is not an Object of the
	 * 			MBean class expected for that role
     *      - a MBean provided for that role does not exist
     */
    public void createRelation(String theRelId,
						       String theRelTypeName,
						       RoleList theRoleList)
				throws RelationServiceNotRegisteredException,
				       IllegalArgumentException,
		               RoleNotFoundException,
		               InvalidRelationIdException,
		               RelationTypeNotFoundException,
		               InvalidRoleValueException
	{
		isActive();

		if (theRelId == null || theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationSupport relSupObj = new RelationSupport(theRelId, objectName,
							   					theRelTypeName, theRoleList);
		validateAndAddRelation(true, relSupObj, null, theRelId,
											theRelTypeName, theRoleList);
    }

    /**
     * Creates a relation type (RelationTypeSupport object) with given role infos
	 * (provided by the RoleInfo objects), and adds it in the Relation Service.
	 *
     * @param theRelTypeName - name of the relation type
     *
     * @param theRoleInfoArray - array of role infos
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception InvalidRelationTypeException - If:
     *           - there is already a relation type with that name
     *           - the same name has been used for two different role infos
     *           - no role info provided
     *           - one null role info provided
     */
    public void createRelationType(String theRelTypeName, RoleInfo[] theRoleInfoArray)
					throws IllegalArgumentException,InvalidRelationTypeException
	{
        if(theRelTypeName == null || theRoleInfoArray == null)
            throw new IllegalArgumentException("Cannot create a RelationType for null values");

		RelationType relType =
			new RelationTypeSupport(theRelTypeName, theRoleInfoArray);

		validateAndAddRelationType(relType);
    }


    /**
     * Retrieves the MBeans associated to given one in a relation.
     * <P>This corresponds to CIM Associators and AssociatorNames operations.
     *
     * @param theMBeanName - ObjectName of MBean
     *
     * @param theRelTypeName - can be null; if specified, only the relations of
	 * 				that type will be considered in the search. Else all
	 * 				relation types are considered.
	 *
     * @param theRoleName - can be null; if specified, only the relations where
	 * 				the MBean is referenced in that role will be considered.
	 * 				Else all roles are considered.
	 *
     * @return an HashMap, where the keys are the ObjectNames of the MBeans
	 * 				associated to given MBean, and the value is, for each key,
	 * 				an ArrayList of the relation ids of the relations where
	 * 				the key MBean is associated to given one (as they can be
	 * 				associated in several different relations).
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     */
    public Map findAssociatedMBeans(ObjectName theMBeanName,
								    String theRelTypeName,
								    String theRoleName)
							throws IllegalArgumentException
	{
		if (theMBeanName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		//1.Find the relations referred by this Relation Type.
		//2.Find the referred MBeans in that Relation.
		//3.Form a HashMap without duplicating the values.

		HashMap referredRelations = (HashMap)
			(findReferencingRelations(theMBeanName,theRelTypeName,theRoleName));

		//The result will be populated in this HashMap
		HashMap toRet = new HashMap();

		Iterator iter = referredRelations.keySet().iterator();
		HashMap referredMBeans = null;
		while(iter.hasNext())
		{
			String relId = (String)(iter.next());
			try
			{
				referredMBeans = (HashMap)(getReferencedMBeans(relId));
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}

			Iterator iter1 = referredMBeans.keySet().iterator();
			while(iter1.hasNext())
			{
				ObjectName mbeanName = (ObjectName)(iter1.next());
				if(! mbeanName.equals(theMBeanName))
				{
					ArrayList relIdList = (ArrayList)(toRet.get(mbeanName));
					if(relIdList == null)
					{
						relIdList = new ArrayList();
						relIdList.add(relId);
						toRet.put(mbeanName,relIdList);
					}
					else
					{
						relIdList.add(relId);
					}

				}
			}
		}

		return toRet;
    }

    /**
     * Retrieves the relations where a given MBean is referenced.
     * <P>This corresponds to the CIM "References" and "ReferenceNames" operations.
     *
     * @param theMBeanName - ObjectName of MBean
     *
     * @param theRelTypeName - can be null; if specified, only the relations of
	 * 				that type will be considered in the search. Else all
	 * 				relation types are considered.
	 *
     * @param theRoleName - can be null; if specified, only the relations where
	 * 				the MBean is referenced in that role will be returned.
	 * 				Else all roles are considered.
	 *
     * @return an HashMap, where the keys are the relation ids of the relations
	 * 				where the MBean is referenced, and the value is, for each
	 * 				key, an ArrayList of role names (as a MBean can be
	 * 				referenced in several roles in the same relation).
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     */
    public Map findReferencingRelations(ObjectName theMBeanName,
										String theRelTypeName,
										String theRoleName)
						throws IllegalArgumentException
	{
        if(theMBeanName == null)
        	throw new IllegalArgumentException("Cannot find Referencing relations for a Null Object");

        HashMap toRet = new HashMap();

        Iterator e = relationsTable.keySet().iterator();
        String relId = null;
        String relTypeName = null;

		try
		{
	        while(e.hasNext())
			{
				//This array List will be the Object for every key in the HashMap.
				ArrayList toPut = new ArrayList();

				relId = (String)(e.next());
				Object relObject = relationsTable.get(relId);

				if(!(theRelTypeName == null) )
				{
					if(relObject instanceof RelationSupport)
					{
						relTypeName = ((RelationSupport)(relObject)).getRelationTypeName();
					}
					else
					{
						relTypeName = (String)(mbeanServer.invoke((ObjectName)(relObject),
		    											  "getRelationTypeName",null,null));
					}
					if(!(relTypeName.equals(theRelTypeName)))
						continue;
				}

				RoleList roleList = null;
				if(relObject instanceof RelationSupport)
				{
					roleList = ((RelationSupport)(relObject)).retrieveAllRoles();
				}
				else
				{
					roleList = (RoleList)(mbeanServer.invoke((ObjectName)relObject,"retrieveAllRoles",
												  		 null,null));
				}
				List roleValueList = null;
				for(int i=0;i<roleList.size();i++)
				{
					Role role = (Role)(roleList.get(i));
					if(theRoleName != null)
					{
						if(role.getRoleName().equals(theRoleName))
						{
							roleValueList = this.getRoleValueFromRoleList(roleList,role.getRoleName());
							for(int j=0;j<roleValueList.size();j++)
							{
								if(((ObjectName)(roleValueList.get(j))).toString().equals(theMBeanName.toString()))
							    	  toPut.add(role.getRoleName());
						    }
	  					}
						continue;
	     			}

		    	   	roleValueList = this.getRoleValueFromRoleList(roleList,role.getRoleName());
				   	for(int j=0;j<roleValueList.size();j++)
					{
				       	if(((ObjectName)(roleValueList.get(j))).toString().equals(theMBeanName.toString()))
						{
							toPut.add(role.getRoleName());
			      	    }
	 			 	}
				}

				if(!( toPut == null || toPut.size() <= 0))
					toRet.put(relId,toPut);
			}//End while loop.
		}
		catch(Exception eee){ }

		return toRet;
    }

    /**
     * Returns the relation ids for relations of the given type.
     *
     * @param theRelTypeName - relation type name
     *
     * @return an ArrayList of relation ids.
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationTypeNotFoundException - if there is no relation
	 * 				type with that name.
     */
    public List findRelationsOfType(String theRelTypeName)
				throws IllegalArgumentException, RelationTypeNotFoundException
	{
		if (theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = getRelationType(theRelTypeName);

        ArrayList toRet = new ArrayList();
		//get the relationIds of those relations that are of the given relation type.
		//add it to the ArrayList and return it.
		Object relObject = null;
		String relTypeName = null;
		String relId = null;

		Iterator  enum = relationsTable.keySet().iterator();
		while(enum.hasNext())
		{
			relId = (String)(enum.next());
			relObject = relationsTable.get(relId);
			if(relObject instanceof RelationSupport)
			{
				relTypeName = ((RelationSupport)(relObject)).getRelationTypeName();
			}
			else
			{
				try
				{
					if(mbeanServer.isRegistered((ObjectName)relObject))
					{
						relTypeName = (String)(mbeanServer.invoke((ObjectName)relObject,"getRelationTypeName",null,null));
					}
				}
				catch(Exception e)
				{
					//e.printStackTrace();
				}
			}

			if(relTypeName != null && relTypeName.equals(theRelTypeName))
			{
				toRet.add(relId);
			}
		}

		//Before returning , check if there was any relation of that name
        return toRet;
    }

    /**
     * Returns all the relation ids for all the relations handled by the
	 * Relation Service
	 *
     * @return ArrayList of String
     */
    public List getAllRelationIds()
	{
		return  new ArrayList(relationsTable.keySet());
    }

    /**
     * Retrieves names of all known relation types
     *
     * @return ArrayList of relation type names (Strings)
     */
    public List getAllRelationTypeNames()
	{
		return new ArrayList(relationTypeObjTable.keySet());
    }

    /**
     * Returns all roles present in the relation
     *
     * @param theRelId - relation id
     *
     * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList (for
	 * 				roles not readable).
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation for given id
     *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
     */
    public RoleResult getAllRoles(String theRelId)
					        throws IllegalArgumentException,
	       						   RelationNotFoundException,
               					   RelationServiceNotRegisteredException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);

		RoleResult result = null;

		if (relObj instanceof RelationSupport)
		{
			result = ((RelationSupport)relObj).getAllRolesFriend(this);
		}
		else
		{
			Object[] params = new Object[0];
			String[] signature = new String[0];

			try
			{
				result = (RoleResult) (mbeanServer.invoke(((ObjectName)relObj),
						  						  "getAllRoles",
												  params,
												  signature));
			}
			catch (Exception exc)
			{
				throw new RuntimeException(exc.getMessage());
			}
		}

		return result;
    }

    /**
     * Returns the flag to indicate if when a notification is received for the
     * unregistration of a MBean referenced in a relation, if an immediate
	 * "purge" of the relations (look for the relations no longer valid) has
	 * to be performed , or if that will be performed only when the
	 * purgeRelations method will be explicitly called. true is immediate purge.
	 *
     * @return boolean indicating the purgeFlag. true - means immediate Purge.
     */
    public boolean getPurgeFlag()
    {
        return this.purgeFlag;
    }//End getPurgeFlag().

    /**
     * Retrieves MBeans referenced in the various roles of the relation.
     *
     * @param theRelId - relation id
     *
     * @return a HashMap mapping:
     *           ObjectName -> ArrayList of String (role names)
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation for given relation id
     */
    public Map getReferencedMBeans(String theRelId)
					throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);
		HashMap toRet = null;

		if (relObj instanceof RelationSupport)
		{
			toRet = (HashMap)(((RelationSupport)relObj).getReferencedMBeans());
		}
		else
		{
			try
			{
				toRet = (HashMap) (mbeanServer.invoke(((ObjectName)relObj),
						"getReferencedMBeans", new Object[0], new String[0]));
			}
			catch (Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return toRet;
    }

   /**
	* Returns name of associated relation type for given relation.
	*
    * @param theRelId - relation id
    *
	* @return  The name of associated relation type for given relation
	*
    * @exception java.lang.IllegalArgumentException - if null parameter
    *
	* @exception RelationNotFoundException - if no relation for given relation id
    */
    public String getRelationTypeName(String theRelId)
					throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);

		String toRet = null;

		if (relObj instanceof RelationSupport)
		{
			toRet = ((RelationSupport)relObj).getRelationTypeName();
		}
		else
		{
			try
			{
				toRet = (String)
				(mbeanServer.invoke(((ObjectName)relObj),"getRelationTypeName",
										new Object[0],new String[0]));
			}
			catch (Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return toRet;
    }

    /**
	 * Retrieves role value for given role name in given relation.
	 *
     * @param theRelId - relation id
     *
     * @param theRoleName - name of role the ArrayList of ObjectName objects
	 * 				being the role value
	 *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation with given id
     *
     * @exception RoleNotFoundException - if:
     *       - there is no role with given name
     *       or
     *       - the role is not readable.
     */
    public List getRole(String theRelId,String theRoleName)
			    		throws RelationServiceNotRegisteredException,
				    		   IllegalArgumentException,
			        	       RelationNotFoundException,
			        	       RoleNotFoundException
	{
		if (theRelId == null || theRoleName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();
		Object relObj = getRelation(theRelId);

		ArrayList toRet = null;

		if (relObj instanceof RelationSupport)
		{
			toRet = (ArrayList)
			(((RelationSupport)relObj).getRoleFriend(theRoleName,false,this));
		}
		else
		{
			Object[] params = new Object[]{theRoleName};
			String[] sig = new String[]{"java.lang.String"};

			try
			{
				toRet = (ArrayList)(mbeanServer.invoke(((ObjectName)relObj),
									"getRole",params,sig));
			}
			catch (MBeanException me)
			{
				if(me.getTargetException() instanceof RoleNotFoundException)
				{
					throw ((RoleNotFoundException)(me.getTargetException()));
				}
				else
				{
					throw new RuntimeException(me.getMessage());
				}
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return toRet;
    }

    /**
     * Retrieves the number of MBeans currently referenced in the given role
     *
     * @param theRelId - relation id
     *
     * @param theRoleName - name of role
     *
     * @return the number of currently referenced MBeans in that role
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation with given id
     *
     * @exception RoleNotFoundException - if there is no role with given name
     */
    public Integer getRoleCardinality(String theRelId,String theRoleName)
								throws IllegalArgumentException,
						               RelationNotFoundException,
						               RoleNotFoundException
	{
		if (theRelId == null || theRoleName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);

		Integer toRet = null;

		if (relObj instanceof RelationSupport)
		{
			toRet = (Integer)(((RelationSupport)relObj).getRoleCardinality(theRoleName));
		}
		else
		{
			Object[] params = new Object[]{theRoleName};
			String[] sig = new String[]{"java.lang.String"};

			try
			{
				toRet = (Integer)(mbeanServer.invoke(((ObjectName)relObj),
						  "getRoleCardinality",params,sig));
			}
			catch (MBeanException me)
			{
				if(me.getTargetException() instanceof RoleNotFoundException)
				{
					throw ((RoleNotFoundException)(me.getTargetException()));
				}
				else
				{
					throw new RuntimeException(me.getMessage());
				}
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return toRet;
    }

    /**
     * Retrieves role info for given role of a given relation type
     *
     * @param theRelTypeName - name of relation type
     *
     * @param theRoleInfoName - name of role
     *
     * @return RoleInfo object.
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationTypeNotFoundException - if the relation type is not
	 * 				known in the Relation Service
	 *
     * @exception RoleInfoNotFoundException - if the role is not part of
	 * 				the relation type.
     */
    public RoleInfo getRoleInfo(String theRelTypeName,String theRoleInfoName)
							throws IllegalArgumentException,
							       RelationTypeNotFoundException,
					               RoleInfoNotFoundException
	{
		if (theRelTypeName == null || theRoleInfoName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = getRelationType(theRelTypeName);
		RoleInfo roleInfo = relType.getRoleInfo(theRoleInfoName);

		return roleInfo;
    }

    /**
     * Retrieves list of role infos (RoleInfo objects) of a given relation type
     *
     * @param theRelTypeName - name of relation type
     *
     * @return ArrayList of RoleInfo.
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationTypeNotFoundException - if there is no relation
	 * 				type with that name.
     */
    public List getRoleInfos(String theRelTypeName)
				throws IllegalArgumentException, RelationTypeNotFoundException
	{
		if (theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = getRelationType(theRelTypeName);

		return relType.getRoleInfos();
    }

    /**
     * Retrieves values of roles with given names in given relation.
     *
     * @param theRelId - relation id
     *
     * @param theRoleNameArray - array of names of roles to be retrieved
     *
     * @return a RoleResult object, including a RoleList (for roles
	 * 				succcessfully retrieved) and a RoleUnresolvedList (for
	 * 				roles not retrieved).
	 *
	 * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception RelationNotFoundException - if no relation with given id
     */
    public RoleResult getRoles(String theRelId,String[] theRoleNameArray)
							throws RelationServiceNotRegisteredException,
							       IllegalArgumentException,
							       RelationNotFoundException
	{
		if (theRelId == null || theRoleNameArray == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();
		Object relObj = getRelation(theRelId);

		RoleResult toRet = null;

		if (relObj instanceof RelationSupport)
		{
			toRet = ((RelationSupport)relObj).getRolesFriend(theRoleNameArray,this);
		}
		else
		{
			try
			{
				Object[] params = new Object[]{theRoleNameArray};
				String[] sig = new String[]{theRoleNameArray.getClass().getName()};

				toRet = (RoleResult)(mbeanServer.invoke(((ObjectName)relObj),"getRoles",params,sig));
			}
			catch (MBeanException me)
			{
				throw new RuntimeException((me.getTargetException()).getMessage());
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return toRet;
    }

    /**
     * Checks if there is a relation identified in Relation Service with
	 * given relation id.
	 *
     * @param theRelId - relation id identifying the relation
     *
     * @return  boolean: true if there is a relation, false else
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     */
    public Boolean hasRelation(String theRelId) throws IllegalArgumentException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		try
		{
			Object result = getRelation(theRelId);
			return new Boolean(true);
		}
		catch (RelationNotFoundException exc)
		{
			return new Boolean(false);
		}
    }

    /**
     * Checks if the Relation Service is active. Current condition is that
	 * the Relation Service must be registered in the MBean Server
	 **
     * @exception RelationServiceNotRegisteredException - if it is not registered
     */
    public void isActive() throws RelationServiceNotRegisteredException
	{
		if (mbeanServer == null)
		{
			throw new RelationServiceNotRegisteredException("Relation Service not registered in the MBean mbeanServer.");
		}
    }

    /**
     * Returns the relation id associated to the given ObjectName if the MBean
	 * has been added as a relation in the Relation Service.
	 *
     * @param theObjName - ObjectName of supposed relation
     *
     * @return relation id (String) or null (if the ObjectName is not a
	 * 				relation handled by the Relation Service)
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     */
    public String isRelation(ObjectName theObjName)
									throws IllegalArgumentException
	{
		if (theObjName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String relId = null;
		relId = (String)(relationObjNamesTable.get(theObjName));

		return relId;
    }

    /**
     * If the relation is represented by a MBean (created by the user and added
	 * as a relation in the Relation Service), returns the ObjectName of the MBean.
	 *
     * @param theRelId - relation id identifying the relation
     *
     * @return ObjectName of the corresponding relation MBean, or null if
	 * 				the relation is not a MBean.
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - there is no relation associated to that id
     */
    public ObjectName isRelationMBean(String theRelId)
					throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);

		if (relObj instanceof ObjectName)
		{
			return ((ObjectName)relObj);
		}
		else
		{
			return null;
		}
    }

    /**
     * Purges the relations.
     * <P>Depending on the purgeFlag value, this method is either called
	 * automatically when a notification is received for the unregistration of
	 * a MBean referenced in a relation (if the flag is set to true), or not
	 * (if the flag is set to false).
	 *
     * <P>In that case it is up to the user to call it to maintain the
	 * consistency of the relations. To be kept in mind that if a MBean is
	 * unregistered and the purge not done immediately, if the ObjectName is
	 * reused and assigned to another MBean referenced in a relation, calling
	 * manually this purgeRelations() method will cause trouble, as will
	 * consider the ObjectName as corresponding to the unregistered MBean,
	 * not seeing the new one.
	 *
     * <P>The behavior depends on the cardinality of the role where the
	 * unregistered MBean is referenced:
     * <P>     - if removing one MBean reference in the role makes its number
	 * 				of references less than the minimum degree, the relation
	 * 				has to be removed.
     * <P>     - if the remaining number of references after removing the
	 * 				MBean reference is still in the cardinality range, keep the
	 * 				relation and update it calling its
	 * 				handleMBeanUnregistration() callback.
	 *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean mbeanServer.
     */
    public void purgeRelations() throws RelationServiceNotRegisteredException
	{
		isActive();

		ArrayList localUnregNtfList = null;

		synchronized(notifList)
		{
			localUnregNtfList = (ArrayList)(notifList.clone());
			notifList = new ArrayList();
		}

		HashMap localMBean2RelIdMap = new HashMap();
		ArrayList obsRefList = new ArrayList();

		synchronized(refMBeansTable)
		{
			for (Iterator unregNtfIter = localUnregNtfList.iterator();
			 									unregNtfIter.hasNext();)
			{
				MBeanServerNotification currNtf =
					(MBeanServerNotification)(unregNtfIter.next());

				ObjectName unregMBeanName = currNtf.getMBeanName();

				obsRefList.add(unregMBeanName);

				HashMap relIdMap = (HashMap)
					(refMBeansTable.get(unregMBeanName));
				localMBean2RelIdMap.put(unregMBeanName, relIdMap);

				refMBeansTable.remove(unregMBeanName);
			}
		}

		updateUnregistrationListener(null, obsRefList);

		for (Iterator unregNtfIter = localUnregNtfList.iterator();
			 										unregNtfIter.hasNext();)
		{
			MBeanServerNotification currNtf =
			(MBeanServerNotification)(unregNtfIter.next());

			ObjectName unregMBeanName = currNtf.getMBeanName();
			HashMap localRelIdMap = (HashMap)
				(localMBean2RelIdMap.get(unregMBeanName));

			if(localRelIdMap != null)
			{
				Set localRelIdSet = localRelIdMap.keySet();

				for (Iterator relIdIter = localRelIdSet.iterator();
				 										relIdIter.hasNext();)
				{
					String currRelId = (String)(relIdIter.next());

					ArrayList localRoleNameList = (ArrayList)
						(localRelIdMap.get(currRelId));

					try
					{
						handleReferenceUnregistration(currRelId,
									  	unregMBeanName, localRoleNameList);
					}
					catch (RelationNotFoundException e)
					{
						throw new RuntimeException(e.getMessage());
					}
					catch (RoleNotFoundException re)
					{
						throw new RuntimeException(re.getMessage());
					}
				}
			}
		}

		return;
    }

    /**
     * Removes given relation from the Relation Service.
     * <P>A RelationNotification notification is sent, its type being:
     * <P>    - RelationNotification.RELATION_BASIC_REMOVAL if the relation
	 * 				was only internal to the Relation Service
     * <P>     - RelationNotification.RELATION_MBEAN_REMOVAL if the relation
	 * 				is registered as a MBean. For MBeans referenced in such
	 * 				relation, nothing will be done,
	 *
     * @param theRelId - relation id of the relation to be removed
     *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
     * @exception  java.lang.IllegalArgumentException - if null parameter
     *
	 * @exception  RelationNotFoundException - if no relation corresponding
	 * 				to given relation id
     */
    public void removeRelation(String theRelId)
			    		throws RelationServiceNotRegisteredException,
						       IllegalArgumentException,
						       RelationNotFoundException
	{
		isActive();

		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object relObj = getRelation(theRelId);

		// Removes it from listener filter
		if (relObj instanceof ObjectName)
		{
			ArrayList obsRefList = new ArrayList();
			obsRefList.add((ObjectName)relObj);
			// Can throw a RelationServiceNotRegisteredException
			updateUnregistrationListener(null, obsRefList);
		}

		sendRelationRemovalNotification(theRelId, null);

		//Let us clean everything here.
		//1. ReferredMBeans Table
		Iterator iter = refMBeansTable.keySet().iterator();
		ArrayList nonreferredObjs = new ArrayList();

		while(iter.hasNext())
		{
			ObjectName refName = (ObjectName)(iter.next());
			//get the relatiosn where the MBean is used.
			HashMap map = (HashMap)(refMBeansTable.get(refName));
			if(map.containsKey(theRelId))
			{
				map.remove(theRelId);
			}
			if(map.isEmpty())
			{
				//This means that the MBean is no longer referred.
				//Add it temporarily to a list
				nonreferredObjs.add(refName);
			}

		}

		Iterator iter1 = nonreferredObjs.iterator();
		while(iter1.hasNext())
		{
			ObjectName mbeanToRemove = (ObjectName)(iter1.next());
			refMBeansTable.remove(mbeanToRemove);
		}

		relationsTable.remove(theRelId);

		if (relObj instanceof ObjectName)
		{
			relationObjNamesTable.remove((ObjectName)relObj);
		}

		String relTypeName = null;

		relTypeName = (String)(relationTypesTable.get(theRelId));
		relationTypesTable.remove(theRelId);
		// - Relation type name to relation id map
		synchronized(relTypeToIDTable)
		{
			ArrayList relIdList = (ArrayList)(relTypeToIDTable.get(relTypeName));
			if (relIdList != null)
			{
				// Can be null if called from removeRelationType()
				relIdList.remove(theRelId);
				if (relIdList.isEmpty())
				{
					// No other relation of that type
					relTypeToIDTable.remove(relTypeName);
				}
			}
		}
    }

    /**
     * Removes given relation type from Relation Service.
     * <P>
     * The relation objects of that type will be removed from the Relation Service.
     *
     * @param theRelTypeName - name of the relation type to be removed
     *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationTypeNotFoundException - If there is no relation
	 * 				type with that name
     */
    public void removeRelationType(String theRelTypeName)
							throws RelationServiceNotRegisteredException,
							       IllegalArgumentException,
							       RelationTypeNotFoundException
	{
		isActive();

		if (theRelTypeName == null)
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = getRelationType(theRelTypeName);

		ArrayList relIdList = null;

		synchronized(relTypeToIDTable)
		{
		    // Note: take a copy of the list as it is a part of a map that
		    //       will be updated by removeRelation() below.
		    ArrayList relIdList1 = (ArrayList) (relTypeToIDTable.get(theRelTypeName));
		    if (relIdList1 != null)
			{
				relIdList = (ArrayList)(relIdList1.clone());
		    }
		}

		// Removes the relation type from all maps
		synchronized(relationTypeObjTable)
		{
		    relationTypeObjTable.remove(theRelTypeName);
		}

		synchronized(relTypeToIDTable)
		{
		    relTypeToIDTable.remove(theRelTypeName);
		}

		// Removes all relations of that type
		if (relIdList != null)
		{
		    for (Iterator relIdIter = relIdList.iterator(); relIdIter.hasNext();)
			{
				String currRelId = (String)(relIdIter.next());
				// Note: will remove it from relationTypesTable :)
				//
				// Can throw RelationServiceNotRegisteredException (detected
				// above)
				// Shall not throw a RelationNotFoundException
				try
				{
				    removeRelation(currRelId);
				}
				catch (RelationNotFoundException exc1)
				{
				    throw new RuntimeException(exc1.getMessage());
				}
		    }
		}
    }

    /**
     * Sends a notification (RelationNotification) for a relation creation.
	 * The notification type is:
     * <P>  - RelationNotification.RELATION_BASIC_CREATION if the relation is
	 * 			an object internal to the Relation Service
     * <P>  - RelationNotification.RELATION_MBEAN_CREATION if the relation is a MBean added
     *   		as a relation. The source object is the Relation Service itself.
     *   		It is called in Relation Service createRelation() and
	 * 			addRelation() methods.
	 *
     * @param theRelId - relation identifier of the updated relation
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception  RelationNotFoundException - if there is no relation
	 * 				for given relation id
     */
    public void sendRelationCreationNotification(String theRelId)
					throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null)
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String relTypeName = (String)(relationTypesTable.get(theRelId));
		ObjectName relObjName = isRelationMBean(theRelId);
		String type = null;

		if(relObjName != null)
		{
			type = RelationNotification.RELATION_MBEAN_CREATION;
		}
		else
		{
			type = RelationNotification.RELATION_BASIC_CREATION;
		}
		RelationNotification notif  = new RelationNotification(type,this,notifNo++,new Date().getTime(),"creating Relation"+theRelId,theRelId,relTypeName,relObjName,null);

		sendNotification(notif);
	}

    /**
     * Sends a notification (RelationNotification) for a relation removal.
	 * The notification type is:
     * <P> - RelationNotification.RELATION_BASIC_REMOVAL if the relation is an
	 * 			object internal to the Relation Service
     * <P> - RelationNotification.RELATION_MBEAN_REMOVAL if the relation is a
	 * 			MBean added as a relation. The source object is the Relation
	 * 			Service itself. It is called in Relation Service removeRelation() method.
	 *
     * @param theRelId - relation identifier of the updated relation
     *
     * @param theUnregMBeanList - ArrayList of ObjectNames of MBeans expected
	 * 				to be unregistered due to relation removal (can be null)
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if there is no relation
	 * 				for given relation id
     */
    public void sendRelationRemovalNotification(String theRelId,
												List theUnregMBeanList)
										throws  IllegalArgumentException,
	       										RelationNotFoundException
	{
		if (theRelId == null)
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String relTypeName = (String)(relationTypesTable.get(theRelId));
		ObjectName relObjName = isRelationMBean(theRelId);
		String type = null;

		if(relObjName != null)
		{
			type = RelationNotification.RELATION_MBEAN_REMOVAL;
		}
		else
		{
			type = RelationNotification.RELATION_BASIC_REMOVAL;
		}

		RelationNotification notif  = new RelationNotification(type, this,
								notifNo++, new Date().getTime(),
								"Removing Relation"+theRelId, theRelId,
								relTypeName, relObjName, theUnregMBeanList);

		sendNotification(notif);
    }

    /**
     * Sends a notification (RelationNotification) for a role update in the
	 * given relation. The notification type is:
     * <P>  - RelationNotification.RELATION_BASIC_UPDATE if the relation is an
	 * 			object internal to the Relation Service
     * <P>  - RelationNotification.RELATION_MBEAN_UPDATE if the relation is a
	 * 			MBean added as a relation.
	 *      	The source object is the Relation Service itself.
     *      	It is called in relation MBean setRole() (for given role) and
	 * 			setRoles() (for each role) methods (implementation provided in
	 * 			RelationSupport class). It is also called in Relation Service
	 * 			setRole() (for given role) and setRoles() (for each role) methods.
	 *
     * @param theRelId - relation identifier of the updated relation
     *
     * @param theNewRole - new role (name and new value)
     *
     * @param theOldRoleValue - old role value (ArrayList of ObjectName objects)
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if there is no relation
	 * 				for given relation id
     */
    public void sendRoleUpdateNotification(String theRelId,
										   Role theNewRole,
										   List theOldRoleValue)
				throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null || theNewRole == null || theOldRoleValue == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String roleName = theNewRole.getRoleName();
		ArrayList newRoleVal = (ArrayList)(theNewRole.getRoleValue());

		String relTypeName = (String)(relationTypesTable.get(theRelId));
		ObjectName relObjName = isRelationMBean(theRelId);
		String type = null;
		if(relObjName != null)
		{
			type = RelationNotification.RELATION_MBEAN_UPDATE;
		}
		else
		{
			type = RelationNotification.RELATION_BASIC_UPDATE;
		}
		RelationNotification notif  = new RelationNotification(type,this,notifNo++,new Date().getTime(),"Updation of a role"+roleName,theRelId,relTypeName,relObjName,roleName,newRoleVal,theOldRoleValue);

		sendNotification(notif);
    }

    /**
     * Sets the flag to indicate if when a notification is received for the
	 * unregistration of a MBean referenced in a relation, if an immediate
	 * "purge" of the relations (look for the relations no longer valid) has
	 * to be performed , or if that will be performed only when the
	 * purgeRelations method will be explicitly called. true is immediate purge.
	 *
     * @param thePurgeFlg - flag
     */
    public void setPurgeFlag(boolean thePurgeFlg)
	{
        this.purgeFlag = thePurgeFlg;
    }// End setPurgeFlag().

    /**
     * Sets the given role in given relation.
     * <P>Will check the role according to its corresponding role definition
	 * provided in relation's relation type.
     * <P>The Relation Service will keep track of the change to keep the
	 * consistency of relations by handling referenced MBean unregistrations.
	 *
     * @param theRelId - relation id
     *
     * @param theRole - role to be set (name and new value)
     *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server.
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation with given id
     *
     * @exception RoleNotFoundException - if:
     *           - internal relation
     *           and
     *           - the role does not exist or is not writable
     *
     * @exception InvalidRoleValueException - if internal relation and value
	 * 				provided for role is not valid:
     *           	- the number of referenced MBeans in given value is less
	 * 					than expected minimum degree
     *           	or
     *           	- the number of referenced MBeans in provided value exceeds
	 * 					expected maximum degree
     *           	or
     *           	- one referenced MBean in the value is not an Object of the
	 * 					MBean class expected for that role
     *           	or
     *           	- a MBean provided for that role does not exist
     *
     * @exception RelationTypeNotFoundException - if unknown relation type
     */
    public void setRole(String theRelId, Role theRole)
				throws  RelationServiceNotRegisteredException,
				        IllegalArgumentException,
				        RelationNotFoundException,
				        RoleNotFoundException,
				        InvalidRoleValueException
	{
		if (theRelId == null || theRole == null)
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();
		Object relObj = getRelation(theRelId);

		if (relObj instanceof RelationSupport)
		{
		    try
			{
				((RelationSupport)relObj).setRoleFriend(theRole,false,this);
		    }
			catch (RelationTypeNotFoundException exc)
			{
				throw new RuntimeException(exc.getMessage());
		    }
		}
		else
		{
		    Object[] params = new Object[]{theRole};
		    String[] sig = new String[]{"javax.management.relation.Role"};

			try
			{
				mbeanServer.invoke(((ObjectName)relObj),"setRole",params,sig);
		    }
			catch(MBeanException me)
		    {
				if(me.getTargetException() instanceof RoleNotFoundException)
				{
					throw (RoleNotFoundException)(me.getTargetException());
				}
				else if(me.getTargetException() instanceof InvalidRoleValueException)
				{
					throw (InvalidRoleValueException)(me.getTargetException());
				}
				else
				{
					throw new RuntimeException(me.getMessage());
				}
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}
    }

    /**
     * Sets the given roles in given relation.
     * <P>Will check the role according to its corresponding role definition
	 * provided in relation's relation type.
	 * <P>The Relation Service keeps track of the changes to keep the
	 * consistency of relations by handling referenced MBean unregistrations.
	 *
     * @param theRelId - relation id
     *
     * @param theRoleList - list of roles to be set
     *
     * @return  a RoleResult object, including a RoleList (for roles
	 * 				succcessfully set) and a RoleUnresolvedList (for roles not set).
	 *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationNotFoundException - if no relation with given id
     */
    public RoleResult setRoles(String theRelId, RoleList theRoleList)
						throws RelationServiceNotRegisteredException,
	       					   IllegalArgumentException,
               				   RelationNotFoundException
	{
		if (theRelId == null || theRoleList == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();
		Object relObj = getRelation(theRelId);

		RoleResult result = null;

		if (relObj instanceof RelationSupport)
		{
			try
			{
				result = ((RelationSupport)relObj).setRolesFriend(theRoleList,true,this);
			}
			catch (RelationTypeNotFoundException e)
			{
				throw new RuntimeException(e.getMessage());
			}

		}
		else
		{
			Object[] params = new Object[]{theRoleList};
			String[] sig = new String[]{"javax.management.relation.RoleList"};

			try
			{
				result = (RoleResult)(mbeanServer.invoke(((ObjectName)relObj),"setRoles",params,sig));
			}
			catch(MBeanException me)
	    	{
				throw new RuntimeException(me.getTargetException().getMessage());
			}
			catch(Exception e)
			{
				throw new RuntimeException(e.getMessage());
			}
		}

		return result;
    }

    /**
     * Handles update of the Relation Service role map for the update of given
	 * role in given relation. It is called in relation MBean setRole() (for
	 * given role) and setRoles() (for each role) methods (implementation
	 * provided in RelationSupport class).
     * <P>It is also called in Relation Service setRole() (for given role) and
	 * setRoles() (for each role) methods.
     * <P>To allow the Relation Service to maintain the consistency (in case of
	 * MBean unregistration) and to be able to perform queries, this method
	 * must be called when a role is updated.
	 *
     * @param theRelId - relation identifier of the updated relation
     *
     * @param theNewRole - new role (name and new value)
     *
     * @param theOldRoleValue - old role value (ArrayList of ObjectName objects)
     *
     * @exception java.lang.IllegalArgumentException - if null parameter
     *
     * @exception RelationServiceNotRegisteredException - if the Relation
	 * 				Service is not registered in the MBean Server
	 *
     * @exception RelationNotFoundException - if no relation for given id.
     */
    public void updateRoleMap(String theRelId, Role theNewRole, List theOldRoleValue)
						throws  IllegalArgumentException,
	       						RelationServiceNotRegisteredException,
               					RelationNotFoundException
	{
		if (theRelId == null || theNewRole == null || theOldRoleValue == null)
		{
	    	throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		// Can throw RelationServiceNotRegisteredException
		isActive();

		// Verifies the relation has been added in the Relation Service
		// Can throw a RelationNotFoundException
		Object result = getRelation(theRelId);

		String roleName = theNewRole.getRoleName();
		ArrayList newRoleValue = (ArrayList)(theNewRole.getRoleValue());
		// Note: no need to test if theOldRoleValue not null before cloning,
		//       tested above.
		ArrayList oldRoleValue = (ArrayList)
		    (((ArrayList)theOldRoleValue).clone());

		// List of ObjectNames of new referenced MBeans
		ArrayList newRefList = new ArrayList();

		for(Iterator newRoleIter = newRoleValue.iterator(); newRoleIter.hasNext();)
		{
		    ObjectName currObjName = (ObjectName)(newRoleIter.next());

		    // Checks if this ObjectName was already present in old value
		    // Note: use copy (oldRoleValue) instead of original
		    //       theOldRoleValue to speed up, as oldRoleValue is decreased
		    //       by removing unchanged references :)
		    int currObjNamePos = oldRoleValue.indexOf(currObjName);

		    if (currObjNamePos == -1)
			{
				// New reference to an ObjectName

				// Stores this reference into map
				// Returns true if new reference, false if MBean already
				// referenced
				boolean isNewFlg = addNewMBeanReference(currObjName,
														theRelId, roleName);

				if (isNewFlg)
				{
				    // Adds it into list of new reference
				    newRefList.add(currObjName);
				}
		    }
			else
			{
				// MBean was already referenced in old value

				// Removes it from old value (local list) to ignore it when
				// looking for remove MBean references
				oldRoleValue.remove(currObjNamePos);
		    }
		}

		// List of ObjectNames of MBeans no longer referenced
		ArrayList obsRefList = new ArrayList();

		// Each ObjectName remaining in oldRoleValue is an ObjectName no longer
		// referenced in new value
		for(Iterator oldRoleIter = oldRoleValue.iterator(); oldRoleIter.hasNext();)
		{
		    ObjectName currObjName = (ObjectName)(oldRoleIter.next());
		    // Removes MBean reference from map
		    // Returns true if the MBean is no longer referenced in any
		    // relation
		    boolean noLongerRefFlg = removeMBeanReference(currObjName,
								  					theRelId, roleName, false);

		    if (noLongerRefFlg)
			{
				// Adds it into list of references to be removed
				obsRefList.add(currObjName);
		    }
		}

		// To avoid having one listener per ObjectName of referenced MBean,
		// and to increase performances, there is only one listener recording
		// all ObjectNames of interest
		updateUnregistrationListener(newRefList, obsRefList);
    }

    /**
	 *	Method inherited from the NotificationListener Interface
	 */
     public void handleNotification(Notification theNtf, Object theHandback)
	 {
		if (theNtf == null)
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		if (theNtf instanceof MBeanServerNotification)
		{
		    String ntfType = theNtf.getType();

		    if (ntfType.equals(
		       MBeanServerNotification.UNREGISTRATION_NOTIFICATION ))
			{
				ObjectName mbeanName =
			    				((MBeanServerNotification)theNtf).getMBeanName();

				if(refMBeansTable.containsKey(mbeanName))
				{
					//This list will be used when purge flag is false and sooner or later, the user calls purge relations.
					//check and add into the NotificationList
					boolean isPresent = false;
					for(int i=0;i<notifList.size();i++)
					{
						if(mbeanName.equals(((MBeanServerNotification)(notifList.get(i))).getMBeanName()))
						{
							isPresent = true;
							break;
						}
					}
					if(!isPresent)
						notifList.add(theNtf);
				}
				if(purgeFlag)
				{
					try
					{
						purgeRelations();
					}
					catch (Exception e)
					{
						throw new RuntimeException(e.getMessage());
					}

					String relId = (String)(relationObjNamesTable.get(mbeanName));

					if (relId != null)
					{
						try
						{
							removeRelation(relId);
						}
						catch (Exception e)
						{
							throw new RuntimeException(e.getMessage());
						}
					}
				}
			}
		}
	}

    /**
     * Returns a NotificationInfo object containing the name of the Java class
     * of the notification and the notification types sent.
     */
    public MBeanNotificationInfo[] getNotificationInfo()
	{
		MBeanNotificationInfo[] notifInfo = new MBeanNotificationInfo[1];

		String[] notifTypes = new String[]{RelationNotification.RELATION_BASIC_CREATION,RelationNotification.RELATION_MBEAN_CREATION,RelationNotification.RELATION_BASIC_UPDATE,RelationNotification.RELATION_MBEAN_UPDATE,RelationNotification.RELATION_BASIC_REMOVAL,RelationNotification.RELATION_MBEAN_REMOVAL};;

		notifInfo[0] = new MBeanNotificationInfo(notifTypes,
								"Relation Notification",
								"Relation Notifications sent during creation/deletion or updation");

		return notifInfo;
    }


////////////////////////////////////////////////////////////////////////////////
	/**
	 *
	 */
    private void validateAndAddRelationType(RelationType theRelTypeObj)
				throws IllegalArgumentException, InvalidRelationTypeException
	{
		if (theRelTypeObj == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String relTypeName = theRelTypeObj.getRelationTypeName();
		try
		{

			RelationType relType = getRelationType(relTypeName);
			if (relType != null)
			{
				throw new InvalidRelationTypeException("RelationType already exists");
			}
		}
		catch (RelationTypeNotFoundException exc)
		{
		}
			relationTypeObjTable.put(new String(relTypeName), theRelTypeObj);
     }


	/**
	 *
	 */
    RelationType getRelationType(String theRelTypeName)
				throws IllegalArgumentException, RelationTypeNotFoundException
	{
		if (theRelTypeName == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		RelationType relType = null;
			relType = (RelationType)(relationTypeObjTable.get(theRelTypeName));

		if (relType == null)
		{
			throw new RelationTypeNotFoundException("No such relation "+theRelTypeName);
		}

		return relType;
    }

	/**
	 *
	 */
    Object getRelation(String theRelId)
				throws IllegalArgumentException, RelationNotFoundException
	{
		if (theRelId == null)
		{
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		Object rel = null;
		rel = relationsTable.get(theRelId);

		if (rel == null)
		{
			throw new RelationNotFoundException("No such Relation"+theRelId);
		}

		return rel;
    }


	/**
	 *
	 */
    private boolean addNewMBeanReference(ObjectName theObjName,
					 					String theRelId, String theRoleName)
			throws IllegalArgumentException
	{
		if (theObjName == null || theRelId == null || theRoleName == null)
		{
	    	throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		boolean isNewFlg = false;

		synchronized(refMBeansTable)
		{
		    HashMap mbeanRefMap = (HashMap)
			(refMBeansTable.get(theObjName));

		    if (mbeanRefMap == null)
			{
				isNewFlg = true;

				ArrayList roleNames = new ArrayList();
				roleNames.add(theRoleName);

				mbeanRefMap = new HashMap();
				mbeanRefMap.put(new String(theRelId), roleNames);

				refMBeansTable.put(theObjName, mbeanRefMap);
		    }
			else
			{
				ArrayList roleNames = (ArrayList)(mbeanRefMap.get(theRelId));

				if (roleNames == null)
				{
				    roleNames = new ArrayList();
				    roleNames.add(theRoleName);

				    mbeanRefMap.put(new String(theRelId), roleNames);
				}
				else
				{
				    roleNames.add(theRoleName);
				}
		    }
		}

		return isNewFlg;
    }

    private boolean removeMBeanReference(ObjectName theObjName,
										 String theRelId,
										 String theRoleName,
										 boolean theAllRolesFlg)
								throws  IllegalArgumentException
	{
		if (theObjName == null || theRelId == null || theRoleName == null)
		{
	    	throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		boolean noLongerRefFlg = false;

		synchronized(refMBeansTable)
		{
		    HashMap mbeanRefMap = (HashMap)
			(refMBeansTable.get(theObjName));

		    if (mbeanRefMap == null)
			{
				return true;
		    }

		    ArrayList roleNames = new ArrayList();
		    if (!theAllRolesFlg)
			{
				roleNames = (ArrayList)(mbeanRefMap.get(theRelId));

				int obsRefIdx = roleNames.indexOf(theRoleName);
				if (obsRefIdx != -1)
				{
				    roleNames.remove(obsRefIdx);
				}
		    }

		    if (roleNames.isEmpty() || theAllRolesFlg)
			{
				mbeanRefMap.remove(theRelId);
		    }

		    if (mbeanRefMap.isEmpty())
			{
				refMBeansTable.remove(theObjName);
				noLongerRefFlg = true;
		    }
		}

		return noLongerRefFlg;
    }

    private void updateUnregistrationListener(List theNewRefList,
											  List theObsRefList)
								throws RelationServiceNotRegisteredException
	{
		if (theNewRefList != null && theObsRefList != null)
		{
	   		if (theNewRefList.isEmpty() && theObsRefList.isEmpty())
		    {
				return;
	    	}
		}

		isActive();

		if (theNewRefList != null || theObsRefList != null)
		{
		    boolean newListenerFlg = false;

			if (notifFilter == null)
			{
				notifFilter = new MBeanServerNotificationFilter();
				newListenerFlg = true;
		    }

		    synchronized(notifFilter)
			{
				if (theNewRefList != null)
				{
			    	for (Iterator newRefIter = theNewRefList.iterator();
				 									newRefIter.hasNext();)
					{
						ObjectName newObjName = (ObjectName)(newRefIter.next());
						notifFilter.enableObjectName(newObjName);
				    }
				}

				if (theObsRefList != null)
				{
				    for (Iterator obsRefIter = theObsRefList.iterator(); obsRefIter.hasNext();)
					{
						ObjectName obsObjName = (ObjectName)
					    (obsRefIter.next());
						notifFilter.disableObjectName(obsObjName);
				    }
				}

				if (newListenerFlg)
				{
				    try
					{
					mbeanServer.addNotificationListener(new ObjectName(
									"JMImplementation:type=MBeanServerDelegate"),
								    this,
								    notifFilter,
								    null);
				    }
					catch (InstanceNotFoundException exc)
					{
						throw new
				       	RelationServiceNotRegisteredException(exc.getMessage());
				    }
					catch(MalformedObjectNameException ee)
				    {
					}
				}
		    }
		}
    }

    private void validateAndAddRelation(boolean theRelBaseFlg,
										RelationSupport theRelObj,
										ObjectName theRelObjName,
										String theRelId,
										String theRelTypeName,
										RoleList theRoleList)
								throws  IllegalArgumentException,
								        RelationServiceNotRegisteredException,
										RoleNotFoundException,
							            InvalidRelationIdException,
							            RelationTypeNotFoundException,
							            InvalidRoleValueException
	{
		if (theRelId == null || theRelTypeName == null ||(theRelBaseFlg &&
	     	(theRelObj == null ||theRelObjName != null)) ||(!theRelBaseFlg &&
	     	(theRelObjName == null ||theRelObj != null)))
		{
	    	throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();

		try
		{
		    Object rel = getRelation(theRelId);

		    if (rel != null)
			{
				throw new InvalidRelationIdException("Relation exists ");
		    }
		}
		catch (RelationNotFoundException exc)
		{
		}

		RelationType relType = getRelationType(theRelTypeName);

		ArrayList roleInfoList = (ArrayList)
		    		(((ArrayList)(relType.getRoleInfos())).clone());

		if(theRoleList == null)
		{
			throw new InvalidRoleValueException();
		}

		if (theRoleList != null)
		{
		    for (Iterator roleIter = theRoleList.iterator(); roleIter.hasNext();)
			{
				Role currRole = (Role)(roleIter.next());
				String currRoleName = currRole.getRoleName();
				ArrayList currRoleValue = (ArrayList)
				    (currRole.getRoleValue());
				RoleInfo roleInfo = null;
				try
				{
				    roleInfo = relType.getRoleInfo(currRoleName);
				}
				catch (RoleInfoNotFoundException exc)
				{
				    throw new RoleNotFoundException(exc.getMessage());
				}

				int problem = getRoleStatusInteger(currRoleName, currRoleValue,
												roleInfo, false, 2).intValue();

				if (problem != 0)
				{
				    if(problem == RoleStatus.NO_ROLE_WITH_NAME ||
				    			problem == RoleStatus.ROLE_NOT_READABLE ||
				    			problem == RoleStatus.ROLE_NOT_WRITABLE )
						throw new RoleNotFoundException();
					else
						throw new InvalidRoleValueException();
				}

				int roleInfoIdx = roleInfoList.indexOf(roleInfo);

				roleInfoList.remove(roleInfoIdx);
		    }
		}

		initialiseMissingRoles(theRelBaseFlg, theRelObj, theRelObjName,
										theRelId, theRelTypeName, roleInfoList);

		synchronized(relationsTable)
		{
		    if (theRelBaseFlg)
			{
				relationsTable.put(new String(theRelId), theRelObj);
		    }
			else
			{
				relationsTable.put(new String(theRelId), theRelObjName);
		    }
		}

		synchronized(relationTypesTable)
		{
		    relationTypesTable.put(new String(theRelId), new String(theRelTypeName));
		}

		synchronized(relTypeToIDTable)
		{
		    ArrayList relIdList = (ArrayList) (relTypeToIDTable.get(theRelTypeName));
		    boolean firstRelFlg = false;
		    if (relIdList == null)
			{
				firstRelFlg = true;
				relIdList = new ArrayList();
		    }
		    relIdList.add(theRelId);
		    if (firstRelFlg)
			{
				relTypeToIDTable.put(new String(theRelTypeName), relIdList);
		    }
		}

		for (Iterator roleIter = theRoleList.iterator(); roleIter.hasNext();)
		{
		    Role currRole = (Role)(roleIter.next());
		    ArrayList dummyList = new ArrayList();

			try
			{
				updateRoleMap(theRelId, currRole, dummyList);
		    }
			catch (RelationNotFoundException exc)
			{
		    }
		}

		try
		{
		    sendRelationCreationNotification(theRelId);
		}
		catch (RelationNotFoundException exc)
		{
		}
    }


	/**
	 *	This method checks the Role for conformance to the RoleInfo.
	 */
    private Integer getRoleStatusInteger(String theRoleName,
										 List theRoleValue,
										 RoleInfo theRoleInfo,
										 boolean writeFlg,int type)
							throws IllegalArgumentException
	{
		if (theRoleName == null || theRoleInfo == null )
		{
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		String roleInfoName = theRoleInfo.getName();

		if (!(theRoleName.equals(roleInfoName)))
		{
		    return new Integer(RoleStatus.NO_ROLE_WITH_NAME);
		}

		if (type == 1)
		{
		    if (!theRoleInfo.isReadable())
			{
				return new Integer(RoleStatus.ROLE_NOT_READABLE);
		    }
			else
			{
				return new Integer(0);
		    }
		}

		if (writeFlg)
		{
		    if (!theRoleInfo.isWritable())
			{
				return new Integer(RoleStatus.ROLE_NOT_WRITABLE);
		    }
		}

		int roleValueLength = theRoleValue.size();

		if (!theRoleInfo.checkMinDegree(roleValueLength))
		{
		    return new Integer(RoleStatus.LESS_THAN_MIN_ROLE_DEGREE);
		}

		if (!theRoleInfo.checkMaxDegree(roleValueLength))
		{
		    return new Integer(RoleStatus.MORE_THAN_MAX_ROLE_DEGREE);
		}

		String className = theRoleInfo.getRefMBeanClassName();

		for (Iterator iter = theRoleValue.iterator(); iter.hasNext();)
		{
		    ObjectName objName = (ObjectName)(iter.next());

		    if(objName == null)
			{
				return new Integer(RoleStatus.REF_MBEAN_NOT_REGISTERED);
		    }

			if (!mbeanServer.isRegistered(objName))
			{
				return new Integer(RoleStatus.REF_MBEAN_NOT_REGISTERED);
		    }

		    try
			{
				if (! mbeanServer.isInstanceOf(objName,className) )
				{
				    return new Integer(RoleStatus.REF_MBEAN_OF_INCORRECT_CLASS);
				}
		    }
			catch (InstanceNotFoundException exc)
			{
				return null;
		    }
		}

		return new Integer(0);
    }


    private void initialiseMissingRoles(boolean theRelBaseFlg,
										RelationSupport theRelObj,
										ObjectName theRelObjName,
										String theRelId,
										String theRelTypeName,
										List theRoleInfoList)
								throws IllegalArgumentException,
								       RelationServiceNotRegisteredException,
								       InvalidRoleValueException
	{
		if ((theRelBaseFlg && (theRelObj == null || theRelObjName != null)) ||
	    		(!theRelBaseFlg && (theRelObjName == null || theRelObj != null)) ||
	    		theRelId == null || theRelTypeName == null || theRoleInfoList == null)
		{
	    	throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
		}

		isActive();

		for (Iterator roleInfoIter = theRoleInfoList.iterator(); roleInfoIter.hasNext();)
		{

		    RoleInfo currRoleInfo = (RoleInfo)(roleInfoIter.next());
		    String roleName = currRoleInfo.getName();


		    ArrayList emptyValue = new ArrayList();

		    Role role = new Role(roleName, emptyValue);

		    if (theRelBaseFlg)
			{
			try {
			    theRelObj.setRoleFriend(role,true,this);

			} catch (RoleNotFoundException e) {
			    throw new RuntimeException(e.getMessage());
			} catch (RelationNotFoundException re) {
			    throw new RuntimeException(re.getMessage());
			} catch (RelationTypeNotFoundException ex) {
			    throw new RuntimeException(ex.getMessage());
			}

		    } else {

			Object[] params = new Object[1];
			params[0] = role;
			String[] signature = new String[1];
			signature[0] = "javax.management.relation.Role";

			try {
			    mbeanServer.invoke(theRelObjName,
						 "setRole",
						 params,
						 signature);

			} catch (InstanceNotFoundException exc1) {
			    throw new RuntimeException(exc1.getMessage());
			} catch (ReflectionException exc3) {
			    throw new RuntimeException(exc3.getMessage());
			} catch (MBeanException exc2) {
			    Exception wrappedExc = exc2.getTargetException();
			    if (wrappedExc instanceof InvalidRoleValueException) {
				throw ((InvalidRoleValueException)wrappedExc);
			    } else {
				throw new RuntimeException(wrappedExc.getMessage());
			    }
			}
		    }
		}
    }

    private void handleReferenceUnregistration(String theRelId,
					       ObjectName theObjName,
					       List theRoleNameList)
	throws IllegalArgumentException,
	       RelationServiceNotRegisteredException,
               RelationNotFoundException,
	       RoleNotFoundException {

	if (theRelId == null ||
	    theRoleNameList == null ||
	    theObjName == null) {
	    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);
	}

	isActive();

	String currRelTypeName = getRelationTypeName(theRelId);

	Object relObj = getRelation(theRelId);


	boolean deleteRelFlg = false;

	for (Iterator roleNameIter = theRoleNameList.iterator();
	     roleNameIter.hasNext();) {

	    if (deleteRelFlg) {
		break;
	    }

	    String currRoleName = (String)(roleNameIter.next());
	    int currRoleRefNbr =
		(getRoleCardinality(theRelId, currRoleName)).intValue();

	    int currRoleNewRefNbr = currRoleRefNbr - 1;

	    RoleInfo currRoleInfo = null;
	    try {
		currRoleInfo = getRoleInfo(currRelTypeName,
					   currRoleName);
	    } catch (RelationTypeNotFoundException ree) {
		throw new RuntimeException(ree.getMessage());
	    } catch (RoleInfoNotFoundException re) {
		throw new RuntimeException(re.getMessage());
	    }

	    boolean chkMinFlg = currRoleInfo.checkMinDegree(currRoleNewRefNbr);

	    if (!chkMinFlg) {
		deleteRelFlg = true;
	    }
	}

	if (deleteRelFlg) {
	    removeRelation(theRelId);
	} else {

	    for (Iterator roleNameIter = theRoleNameList.iterator();
		 roleNameIter.hasNext();) {

		String currRoleName = (String)(roleNameIter.next());

		if (relObj instanceof RelationSupport) {
		    try {
			((RelationSupport)relObj).handleMBeanUnregistrationFriend(
						  theObjName,
						  currRoleName,
						  this);
		    } catch (RelationTypeNotFoundException exc3) {
			throw new RuntimeException(exc3.getMessage());
		    } catch (InvalidRoleValueException exc4) {
			throw new RuntimeException(exc4.getMessage());
		    }

		} else {
		    // Relation MBean
		    Object[] params = new Object[2];
		    params[0] = theObjName;
		    params[1] = currRoleName;
		    String[] signature = new String[2];
		    signature[0] = "javax.management.ObjectName";
		    signature[1] = "java.lang.String";
		    try {
			mbeanServer.invoke(((ObjectName)relObj),
					     "handleMBeanUnregistration",
					     params,
					     signature);
		    } catch (InstanceNotFoundException exc1) {
			throw new RuntimeException(exc1.getMessage());
		    } catch (ReflectionException exc3) {
			throw new RuntimeException(exc3.getMessage());
		    } catch (MBeanException exc2) {
			Exception wrappedExc = exc2.getTargetException();
			throw new RuntimeException(wrappedExc.getMessage());
		    }
			}
	    }
	}

	return;
    }

    private  void checkRoleInfoConsistency(RoleInfo[] theRoleInfoArray)
			throws InvalidRelationTypeException
	{
		if(!( theRoleInfoArray.length >0))
			throw new InvalidRelationTypeException("No elements in the RoleInfo");

		if(! ( this.validateRoleInfoArray(theRoleInfoArray) ) )
			throw new InvalidRelationTypeException("ERR_VALUE");
	}

	private boolean validateRoleInfoArray(RoleInfo [] info){

		if(info == null){
			ERR_VALUE = "Cannot create RelationType for null RoleInfo";
			return false;
		}
		Hashtable temp = new Hashtable();
		for(int i=0;i<info.length;i++){
			if(info[i] == null){
				ERR_VALUE="Cannot create RelationType for null RoleInfo at"+ i +" st Index";
				return false;
			}
			else{
				if(! (temp.containsKey( ((RoleInfo)info[i]).getName() ) ) ){
					temp.put( ((RoleInfo)info[i]).getName(),new Object() );
				}
				else{
					ERR_VALUE="Cannot create RelationType duplicate RoleInfos";
					return false;
				}
			}
		}
		return true;
	}

    private boolean isRelationIdAlreadyExists(String relId){
        Iterator e = relationsTable.keySet().iterator();
        boolean isPresent = false;
        while(e.hasNext()){
            if( relId.equals( (String)(e.next()) ) ){
            isPresent = true;
            break;
            }
        }
        return isPresent;
    }

	private List getRoleValueFromRoleList(RoleList roleList,String roleName){

		for(int i=0;i<roleList.size();i++){
			Role temp = (Role)(roleList.get(i));
			if(temp.getRoleName().equals(roleName)){
				return temp.getRoleValue();
			}
		}
		return null;
	}
}