/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

/**
 * A RoleList represents a list of roles (Role objects). It is used as
 * parameter when creating a relation, and when trying to set several roles
 * in a relation (via 'setRoles()' method). It is returned as part of a
 * RoleResult, to provide roles successfully retrieved.
 */
public class RoleList extends ArrayList
{
	/**
	 * Constructs an empty RoleList.
	 */
	public RoleList()
	{
		super();
	}

	/**
	 * Constructs an empty RoleList with the initial capacity specified.
	 *
	 * @param theInitialCapacity - initial capacity
	 */
	public RoleList(int theInitialCapacity)
	{
		super(theInitialCapacity);
	}

	/**
	 * Constructs a RoleList containing the elements of the ArrayList specified,
	 * in the order in which they are returned by the ArrayList's iterator.
	 * The RoleList instance has an initial capacity of 110% of the size of
	 * the ArrayList specified.
	 *
	 * @param theList - list of Role objects
	 *
	 * @exception java.lang.IllegalArgumentException - if:
	 * 				- null parameter or
	 *           	- an element in the ArrayList is not a Role
	 */
	public RoleList(List theList) throws IllegalArgumentException
	{
		if(theList == null)
			throw new IllegalArgumentException("Cannot create RoleList with null parameter!!!");

		Iterator iter = theList.iterator();
		int i=0;

		while(iter.hasNext())
		{
			Object temp = iter.next();
			i += 1;
			super.add(temp);

			if(!(temp instanceof Role))
			   throw new IllegalArgumentException("Element at "+i+" index is not a Role ");
		}
	}

	/**
	 * Inserts the role specified as an element at the position specified.
	 * Elements with an index greater than or equal to the current position
	 * are shifted up.
	 *
	 * @param theIndex - The position in the list where the new Role object
	 * 				is to be inserted.
	 *
	 * @param theRole - The Role object to be inserted.
	 *
	 * @throws java.lang.IllegalArgumentException - if the role is null.
	 *
	 * @throws java.lang.IndexOutOfBoundsException - if accessing with an
	 * 				index outside of the list.
	 */
	public void add(int theIndex, Role theRole)
         				throws  IllegalArgumentException,
								IndexOutOfBoundsException
	{
		if(theRole == null)
			throw new IllegalArgumentException();

		// We dont have to specifically throw IndexOutOfBoundsException as it will be thrown
		// in the ArrayList.add()
		super.add(theIndex,theRole);
	}

	/**
	 * Adds the Role specified as the last element of the list.
	 *
	 * @param theRole - the role to be added.
	 *
	 * @exception java.lang.IllegalArgumentException - if the role is null.
	 */
	public void add(Role theRole) throws IllegalArgumentException
	{
		if(theRole == null)
			throw new IllegalArgumentException();

		super.add(theRole);
		//this.roleList.add(theRole);
	}

	/**
	 * Inserts all of the elements in the RoleList specified into this list,
	 * starting at the specified position, in the order in which they are
	 * returned by the Iterator of the RoleList specified.
	 *
	 * @param theIndex - Position at which to insert the first element from
	 * 				the RoleList specified.
	 *
	 * @param theRoleList - Elements to be inserted into the list.
	 *
	 * @throws java.lang.IllegalArgumentException - if the role is null.
	 *
	 * @throws java.lang.IndexOutOfBoundsException - if accessing with an
	 * 				index outside of the list.
	 */
	public boolean addAll(int theIndex, RoleList theRoleList)
               			throws  IllegalArgumentException,
					  			IndexOutOfBoundsException
	{
		if(theRoleList == null)
			throw new IllegalArgumentException();

		// We dont have to specifically throw IndexOutOfBoundsException as
		// it will be thrown in the ArrayList.addAll()
		return super.addAll(theIndex, theRoleList);
	}

	/**
	 * Appends all the elements in the RoleList specified to the end of the
	 * list, in the order in which they are returned by the Iterator of the
	 * RoleList specified.
	 *
	 * @param theRoleList - Elements to be inserted into the list (can be null)
	 *
	 * @throws java.lang.IndexOutOfBoundsException - if accessing with an
	 * 				index outside of the list.
	 */
	public boolean addAll(RoleList theRoleList) throws IndexOutOfBoundsException
	{
		// We dont have to specifically throw IndexOutOfBoundsException as
		// it will be thrown in the ArrayList.addAll()
		if(theRoleList == null)
			return true;

		return super.addAll(theRoleList);
	}

	/**
	 * Cloning. This will create a duplicate copy of the RoleList.
	 *
	 * @return  Returns a new RoleList, but no copy of the included elements.
	 *
	 * Overrides:
	 *           clone in class java.util.ArrayList
	 */
	public java.lang.Object clone()
	{
		return new RoleList( (ArrayList)(super.clone()) );
	}

	/**
	 * Sets the element at the position specified to be the role specified.
	 * The previous element at that position is discarded.
	 *
	 * @param theIndex - The position specified.
	 *
	 * @param theRole - The value to which the role element should be set.
	 *
	 * @throws java.lang.IllegalArgumentException - if the role is null.
	 *
	 * @throws java.lang.IndexOutOfBoundsException - if accessing with an
	 * 				index outside of the list.
	 */
	public void set(int theIndex, Role theRole)
					         throws IllegalArgumentException,
									IndexOutOfBoundsException
	{
		if(theRole == null)
			throw new IllegalArgumentException();

		// We dont have to specifically throw IndexOutOfBoundsException as it will be thrown
		// in the ArrayList.set()

		super.set(theIndex,theRole);
	}
}